### ############################################################################
### sshd-lib.pl
###
### Ryan W. Maple <ryan@guardiandigital.com>
### Guardian Digital, Inc.
### ############################################################################

  ### Include the general subroutines from web-lib.pl and do init_config().
  do '/usr/webtool/web-lib.pl';
  &init_config();

  ### ###############################################################
  ### AddACL()
  ### ###############################################################
  ### Used to delete an entry from /etc/hosts.(allow|deny).
  ### ###############################################################

  sub AddACL() {
    local ($host, $access, $service);
    ($host, $access, $service) = @_;


    ### Validate input.
    if ($access eq "Allow") { $access = "allow"; }
    elsif ($access eq "Deny") { $access = "deny"; }
    else { return 0; }

    ### Get list of all IPs for each service.
    @sshd   = &GetACL('allow', 'sshd');
    @simap  = &GetACL('allow', 'simap');
    @spop3  = &GetACL('allow', 'spop3');

    ### Build line.
    $sshd = "sshd: ";   foreach (@sshd)  { $sshd  .= "$_ "; }
    $simap = "simap: "; foreach (@simap) { $simap .= "$_ "; }
    $spop3 = "spop3: "; foreach (@spop3) { $spop3 .= "$_ "; }

    ### Append new address to the appropriate service.
    $$service .= "$host";

    ### Write the updated services back to disk.
    $WROTE_sshd = 0; $WROTE_simap = 0; $WROTE_spop3 = 0;
    $lref = &read_file_lines("/etc/hosts.$access");
      foreach $line (@$lref) {
        if ($line =~ m/^sshd/)     { $line = $sshd;  $WROTE_sshd  = 1; }
        elsif ($line =~ m/^simap/) { $line = $simap; $WROTE_simap = 1; }
        elsif ($line =~ m/^spop3/) { $line = $spop3; $WROTE_spop3 = 1; }
      }

      if (!$WROTE_sshd)  { push @$lref, $sshd; }
      if (!$WROTE_simap) { push @$lref, $simap; }
      if (!$WROTE_spop3) { push @$lref, $spop3; }
    &flush_file_lines;

  }

  ### ###############################################################
  ### DeleteACL()
  ### ###############################################################
  ### Used to delete an entry from /etc/hosts.(allow|deny).
  ### ###############################################################

  sub DeleteACL() {
    local ($host, $access, $service);
    ($host, $access, $service) = @_;


    ### Validate input.
    if ($access !~ m/(allow|deny)/) { return 0; }

    ### Get list of all IPs for each service.
    @sshd   = &GetACL('allow', 'sshd');
    @simap  = &GetACL('allow', 'simap');
    @spop3  = &GetACL('allow', 'spop3');


    ### Build lines.
    $sshd = "sshd: ";
    foreach (&GetACL('allow', 'sshd')) {
      if ($service eq "sshd") { if ($_ ne $host) { $sshd .= "$_ "; } }
      else { $sshd .= "$_ "; }
    }

    $simap = "simap: ";
    foreach (&GetACL('allow', 'simap')) {
      if ($service eq "simap") { if ($_ ne $host) { $simap .= "$_ "; } }
      else { $simap .= "$_ "; }
    }

    $spop3 = "spop3: ";
    foreach (&GetACL('allow', 'spop3')) {
      if ($service eq "spop3") { if ($_ ne $host) { $spop3 .= "$_ "; } }
      else { $spop3 .= "$_ "; }
    }


    ### Write the updated services back to disk.
    $lref = &read_file_lines("/etc/hosts.$access");
      foreach $line (@$lref) {
        if ($line =~ m/^sshd/)     { $line = $sshd; }
        elsif ($line =~ m/^simap/) { $line = $simap; }
        elsif ($line =~ m/^spop3/) { $line = $spop3; }
      }
    &flush_file_lines;

  }


  ### ###############################################################
  ### GetACL()
  ### ###############################################################
  ### Used to return all the hosts for a specified service.
  ### ###############################################################

  sub GetACL() {
    my ($access, $service) = @_;
    my @services;

    $lref = &read_file_lines("/etc/hosts.$access");
      foreach $line (@$lref) {
        if ($line =~ m/^$service\:\s+(.*)$/) { @services = split /\s+/, $1; }
      }
    &flush_file_lines("/etc/hosts.$access");

    return @services;
  }


  ### ###############################################################
  ### PrintError()
  ### ###############################################################
  ### Used to pop up a JavaScript with a $message then return user to 
  ### previous page.
  ### ###############################################################

  sub PrintError {
    my ($message) = @_;

    print <<ErrorEOF;
Content-type: text/html


<HTML>
<HEAD>
  <TITLE> </TITLE>

  <SCRIPT LANGUAGE="JavaScript">
    alert("$message");
    history.go(-1);
  </SCRIPT>
</HEAD>
</HTML>  
ErrorEOF
  }


  ### ###############################################################
  ### GetAdminEmail()
  ### ###############################################################
  ### Extract the admin email from /etc/logsum.conf.
  ### ###############################################################

  sub GetAdminEmail {
    my ($email);

    &obtain_lock("/etc/logsum.conf");
    open LOGSUM, "/etc/logsum.conf";
      while (<LOGSUM>) { if ($_ =~ m/^EMAIL\ \=\ (.*)$/) { $email = $1; }}
    close LOGSUM;
    &release_lock("/etc/logsum.conf");

    if ($email) { return $email; }
    else        { return "No Address Defined"; }
  }


  ### ####################################################
  ### Return one like a good little library *pat on head*.
  ### ####################################################
  1;

