#!/usr/bin/perl

  ### Do startup stuff.
  require "./gdbackup-lib.pl";
  &ReadParse();

  ### Declare some variables.
  my (%fileBackups, @myBackups, $backupFile);
  $backupFile = "lang/en";

  ### ####################################### ###
  ### Time for some good ol' sanity checking. ###
  ### ####################################### ###
  if (!$in{'backup_name'})    { &PrintError("Please enter a name."); exit; }
  if (!$in{'backup_fhandle'}) {
    &PrintError("Please enter a filehandle."); exit;
  }
  if (!$in{'backup_include'}) {
    &PrintError("Please enter at least one file or directory to include.");
    exit;
  }


  if ($in{'backup_fhandle'} !~ m/^[A-Za-z0-9\_\.]+$/) {
    &PrintError("'$in{'backup_fhandle'}' is an invalid filehandle.");
    exit;
  }

  if ($in{'backup_fhandle'} eq $in{'backup_name'}) {
    &PrintError("The 'Backup Label' cannot be the same as the 'Filename'.");
    exit;
  }

  ### Read the file in, then iterate and pull out existing backups.  Also make
  ### sure that the 'file' or the 'name' are not duplicates...
  &read_file("$backupFile", \%fileBackups);
  foreach $x (sort keys %fileBackups) {

    ### Duplicate 'name', complain.
    if (($x =~ m/\_name$/) && ($fileBackups{$x} eq $in{'backup_name'})) {
      &PrintError("The Label '$in{'backup_name'}' is already assigned.  Please choose another.");
      exit;
    }

    ### Duplicate 'filename', complain.
    elsif (($x =~ m/\_file$/) && ($fileBackups{$x} eq $in{'backup_fhandle'})) {
      &PrintError("The Filename '$in{'backup_fhandle'}' is already assigned.  Please choose another.");
      exit;
    }

    if ($x =~ m/ub([0-9]+)_name/) {
      $name = "ub$1_name";
      $file = "ub$1_file";
      $args = "ub$1_args";

      $myBackups[$1]->{'name'} = $name;
      $myBackups[$1]->{'file'} = $file;
      $myBackups[$1]->{'args'} = $args;
    }
  }



  ### Count backups and increment, then format into a two-digit integer.
  for ($i = 1; $i < @myBackups; $i++) { }
  if ($i < 10) { $i = "0$i"; }


  ### Lock and open the output file.
  &obtain_lock($backupFile);
  open(BACKUPFILE, ">>$backupFile");

    ### Print the lines to the file. 
    print BACKUPFILE "\nub$i\_name=$in{'backup_name'}\n";
    print BACKUPFILE "ub$i\_file=$in{'backup_fhandle'}\n";

    if ($in{'backup_exclude'}) {
      @excludes = split /\ /, $in{'backup_exclude'};
      foreach $exclude (@excludes) { $excludeStr .= "--exclude $exclude"; }
    
      print BACKUPFILE "ub$i\_args=$excludeStr $in{'backup_include'}\n";
    }
    else { print BACKUPFILE "ub$i\_args=$in{'backup_include'}\n"; }

  ### Close and unlock the output file.
  close(BACKUPFILE);
  &release_lock($backupFile);

  ### I'm... sailing... away...
  &redirect("");
