# bind8-lib.pl
# Common functions for bind8 config files

  do '/usr/webtool/web-lib.pl';
  &init_config();
  do './records-lib.pl';

### #########################################################################
### get_config()
### Returns an array of references to assocs, each containing the details of
### one directive
### #########################################################################
sub get_config {
  if (!@get_config_cache) {
    $confpath = $config{'chroot'} . $config{'named_conf'};
    @get_config_cache = &read_config_file($config{'named_conf'});
  }

  return \@get_config_cache;
}

### #########################################################################
### get_config_parent()
### Returns a structure containing the top-level config as members
### #########################################################################
sub get_config_parent {
  return { 'file' => $config{'named_conf'},
           'type' => 1,
           'line' => 0,
           'eline' => $lines_count{$config{'named_conf'}},
           'members' => &get_config() };
}


### ##################################################
### read_config_file(file, [expand includes])
### Reads a config file and returns an array of values
### ##################################################
sub read_config_file {
  local($lnum, $line, $cmode, @ltok, @lnum, @tok, @rv, $i, $t, $j, $ifile, 
        @inc, $str);

  $lnum = 0;
  open(FILE, $config{'chroot'}.$_[0]);

  while($line = <FILE>) {
    # strip comments
    $line =~ s/\r|\n//g;
    $line =~ s/#.*$//g;
    $line =~ s/\/\/.*$//g;
    $line =~ s/\/\*.*\*\///g;

    while(1) {
      if (!$cmode && $line =~ /\/\*/) {
        # start of a C-style comment
        $cmode = 1;
        $line =~ s/\/\*.*$//g;
      }
      elsif ($cmode) {
        if ($line =~ /\*\//) {
          # end of comment
          $cmode = 0;
          $line =~ s/^.*\*\///g;
        }
        else { $line = ""; last; }
      }
      else { last; }
    }

    # split line into tokens
    undef(@ltok);
    while(1) {
      if ($line =~ /^\s*\"([^"]*)"(.*)$/) { push(@ltok, $1); $line = $2; }
      elsif ($line =~ /^\s*([{};])(.*)$/) { push(@ltok, $1); $line = $2; }
      elsif ($line =~ /^\s*([^{}; \t]+)(.*)$/) { push(@ltok, $1); $line = $2; }
      else { last; }
    }

    foreach $t (@ltok) {
      push(@tok, $t); push(@lnum, $lnum);
    }

    $lnum++;
  }

  close(FILE);
  $lines_count{$_[0]} = $lnum;

  # parse tokens into data structures
  $i = 0; $j = 0;
  while($i < @tok) {
    $str = &parse_struct(\@tok, \@lnum, \$i, $j++, $_[0]);
    if ($str) { push(@rv, $str); }
  }

  # expand include directives
  for($i=0; $i<@rv; $i++) {
    if (lc($rv[$i]->{'name'}) eq "include" && !$_[1]) {
      # found one.. replace the include directive with it
      $ifile = $rv[$i]->{'value'};

      if ($ifile !~ /^\//) { $ifile = &base_directory(\@rv)."/$ifile"; }
      @inc = &read_config_file($ifile, 1);

      # update index of included structures
      for($j=0; $j<@inc; $j++) { $inc[$j]->{'index'} += $rv[$i]->{'index'}; }

      # update index of structures after include
      for($j=$i+1; $j<@rv; $j++) { $rv[$j]->{'index'} += scalar(@inc) - 1; }

      splice(@rv, $i--, 1, @inc);
    }
  }

  return @rv;
}

### ############################################################
### parse_struct(&tokens, &lines, &line_num, index, file)
### A structure can either have one value, or a list of values.
### Pos will end up at the start of the next structure
### ############################################################
sub parse_struct {
  local(%str, $i, $t, @vals, $str);
  $i = ${$_[2]};

  $str{'name'} = lc($_[0]->[$i]);
  $str{'line'} = $_[1]->[$i];

#print "name = $str{'name'}\n";
  while(1) {
    $t = $_[0]->[++$i];
#print "  t = $t\n";
    if ($t eq "{" || $t eq ";" || $t eq "}") { last; }
    elsif (!defined($t)) { ${$_[2]} = $i; return undef; }
    else { push(@vals, $t); }
  }

  $str{'values'} = \@vals;
  $str{'value'} = $vals[0];
  $str{'index'} = $_[3];
  $str{'file'} = $_[4];


#print "  value = $str{'value'}\n";
#print "  FINAL _t_= $t\n\n";

  if ($t eq "{") {
    ### contains sub-structures.. parse them
    local(@mems, $j);
    $i++;		# skip '{'
    $str{'type'} = 1;
    $j = 0;

    while($_[0]->[$i] ne "}") {
      if (!defined($_[0]->[$i])) { ${$_[2]} = $i; return undef; }
      $str = &parse_struct($_[0], $_[1], \$i, $j++, $_[4]);
      if ($str) { push(@mems, $str); }
    }

    $str{'members'} = \@mems;
    $i += 2;				### skip trailing '}' and ';'
  }
  else {
    # only a single value..
    $str{'type'} = 0;
    if ($t eq ";") { $i++; }		### Skip trailing ';'
  }

#print "<HR>\n";

  ### ending line is the line number the trailing ';' is on.
  $str{'eline'} = $_[1]->[$i-1];

  ${$_[2]} = $i;

  return \%str;
}

### ##################
### find(name, &array)
### ##################
sub find {
  local($c, @rv);

  foreach $c (@{$_[1]}) { if ($c->{'name'} eq $_[0]) { push(@rv, $c); } }

  return @rv ? wantarray ? @rv : $rv[0] : wantarray ? () : undef;
}


# find_value(name, &array)
sub find_value
{
local(@v);
@v = &find($_[0], $_[1]);
if (!@v) { return undef; }
elsif (wantarray) { return map { $_->{'value'} } @v; }
else { return $v[0]->{'value'}; }
}

# base_directory([&config])
# Returns the base directory for named files
sub base_directory
{
local($opts, $dir, $conf);
$conf = $_[0] ? $_[0] : &get_config();
if (($opts = &find("options", $conf)) &&
    ($dir = &find("directory", $opts->{'members'}))) {
	return $dir->{'value'};
	}
$config{'named_conf'} =~ /^(.*)\/[^\/]+$/;
return $1;
}

### save_directive(&parent, name, &values, indent, [structonly])
### Given a structure containing a directive name, type, values and members
### add, update or remove that directive in config structure and data files.
### Updating of files assumes that there is no overlap between directives -
### each line in the config file must contain part or all of only one directive.
sub save_directive {
  local(@oldv, @newv, $pm, $i, $o, $n, $lref, @nl);
  $pm = $_[0]->{'members'};
  @oldv = &find($_[1], $pm);
  @newv = @{$_[2]};

  for($i=0; $i<@oldv || $i<@newv; $i++) {
    if ($i >= @oldv) {
      ### a new directive is being added.. put it at the end of
      ### the parent
      if (!$_[4]) {
        $lref = &read_file_lines($config{'chroot'}.$_[0]->{'file'});
        @nl = &directive_lines($newv[$i], $_[3]);
        splice(@$lref, $_[0]->{'eline'}, 0, @nl);
        $newv[$i]->{'file'} = $_[0]->{'file'};
        $newv[$i]->{'line'} = $_[0]->{'eline'};
        $newv[$i]->{'eline'} =
        $_[0]->{'eline'} + scalar(@nl) - 1;
        &renumber(&get_config(), $_[0]->{'eline'},
        $_[0]->{'file'}, scalar(@nl));
      }

      push(@$pm, $newv[$i]);
    }
    elsif ($i >= @newv) {
      ### a directive was deleted
      if (!$_[4]) {
        $lref = &read_file_lines($config{'chroot'}.$oldv[$i]->{'file'});
        $ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
        splice(@$lref, $oldv[$i]->{'line'}, $ol);
        &renumber(&get_config(), $oldv[$i]->{'eline'},
        $oldv[$i]->{'file'}, -$ol);
      }

      splice(@$pm, &indexof($oldv[$i], @$pm), 1);
    }
else {
# updating some directive
if (!$_[4]) {
$lref = &read_file_lines($config{'chroot'}.$oldv[$i]->{'file'});
@nl = &directive_lines($newv[$i], $_[3]);
$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
splice(@$lref, $oldv[$i]->{'line'}, $ol, @nl);
$newv[$i]->{'file'} = $_[0]->{'file'};
$newv[$i]->{'line'} = $oldv[$i]->{'line'};
$newv[$i]->{'eline'} =
$oldv[$i]->{'line'} + scalar(@nl) - 1;
&renumber(&get_config(), $oldv[$i]->{'eline'},
$oldv[$i]->{'file'}, scalar(@nl) - $ol);
}
$pm->[&indexof($oldv[$i], @$pm)] = $newv[$i];
}
  }
}

# directive_lines(&directive, tabs)
# Renders some directive into a number of lines of text
sub directive_lines
{
local(@rv, $v, $m, $i);
$rv[0] = "\t" x $_[1];
$rv[0] .= "$_[0]->{'name'}";
foreach $v (@{$_[0]->{'values'}}) {
	if ($need_quote{$_[0]->{'name'}} && !$i) { $rv[0] .= " \"$v\""; }
	else { $rv[0] .= " $v"; }
	$i++;
	}
if ($_[0]->{'type'}) {
	# multiple values.. include them as well
	$rv[0] .= " {";
	foreach $m (@{$_[0]->{'members'}}) {
		push(@rv, &directive_lines($m, $_[1]+1));
		}
	push(@rv, ("\t" x ($_[1]+1))."}");
	}
$rv[$#rv] .= ";";
return @rv;
}

# renumber(&directives, line, file, count)
# Runs through the given array of directives and increases the line numbers
# of all those greater than some line by the given count
sub renumber
{
local($d);
foreach $d (@{$_[0]}) {
	if ($d->{'file'} eq $_[2]) {
		if ($d->{'line'} > $_[1]) { $d->{'line'} += $_[3]; }
		if ($d->{'eline'} > $_[1]) { $d->{'eline'} += $_[3]; }
		}
	if ($d->{'type'}) {
		&renumber($d->{'members'}, $_[1], $_[2], $_[3]);
		}
	}
}


### choice_input(text, name, &config, [display, option]+)
sub choice_input {
  local($rv, $v, $i, @ops);
  $rv  = "<td><DIV CLASS=nbody><b>$_[0]</b></td>\n";
  $rv .= "<td valign=top><DIV CLASS=nbody>";
  $v = &find_value($_[1], $_[2]);

  for($i=3; $i<@_; $i+=2) {
    $rv .= sprintf "<input type=radio name=%s value='%s' %s> %s\n",
                   $_[1], $_[$i+1], $v eq $_[$i+1] ? "checked" : "", $_[$i];
  }

  return $rv."</td>\n";
}



### save_choice(name, &parent, indent)
sub save_choice {
  local($nd);
  if ($in{$_[0]}) { $nd = { 'name' => $_[0], 'values' => [ $in{$_[0]} ] }; }

  &save_directive($_[1], $_[0], $nd ? [ $nd ] : [ ], $_[2]);
}


#### addr_match_input(text, name, &config)
#### A field for editing a list of addresses, ACLs and partial IP addresses
sub addr_match_input {
  local($v, $rv, $av, @av);
  my $DEFAULT_BEHAVIOR = 0;

  $v = &find($_[1], $_[2]);

  foreach $av (@{$v->{'members'}}) { push(@av, $av->{'name'}); }

  $rv  = "<td valign=top>\n";
  $rv .= "  <DIV CLASS=nbody><b>$_[0]</b>\n";
  $rv .= "  <BR><BR><input type=radio name=$_[1]_def value=1 ";


  if ($av[0] eq $_[4]) {
    $rv .= "checked> $_[3]";
    $rv .= "<BR><input type=radio name=$_[1]_def value=0> $text{'listed'}<br>";
    $DEFAULT_BEHAVIOR = 1;
  }
  else {
    $rv .= "> $_[3]";
    $rv .= "<BR><input type=radio name=$_[1]_def value=0 CHECKED> ";
    $rv .= "$text{'listed'}<br>";
  }

  $rv .= "</DIV><DIV CLASS=tbody><textarea name=$_[1] rows=3 cols=15>";
    if (!$DEFAULT_BEHAVIOR) { $rv .= join("\n", @av); }
  $rv .= "</textarea></td>\n";
}


sub save_addr_match {
  local($addr, @vals, $dir);
  if ($in{"$_[0]_def"}) { &save_directive($_[1], $_[0], [ ], $_[2]); }
  else {
    foreach $addr (split(/\s+/, $in{$_[0]})) {push(@vals, { 'name' => $addr });}
    $dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
    &save_directive($_[1], $_[0], [ $dir ], $_[2]);
  }
}

# address_input(text, name, &config, type)
sub address_input {
  local($v, $rv, $av, @av);
  $v = &find($_[1], $_[2]);
  foreach $av (@{$v->{'members'}}) { push(@av, $av->{'name'}); }

  if ($_[3] == 0) {
    ### text area
    $rv  = "<td valign=top><DIV CLASS=nbody><b>$_[0]</b></td> <td valign=top>";
    $rv .= "<textarea name=$_[1] rows=3 cols=15>".
    	join("\n", @av)."</textarea></td>\n";
  }
  else {
    $rv = "<td valign=top><b>$_[0]</b></td> <td colspan=3 valign=top>";
    $rv .= "<input name=$_[1] size=50 value=\"".join(' ',@av)."\"></td>\n";
  }

  return $rv;
}

sub save_address
{
local($addr, @vals, $dir);
foreach $addr (split(/\s+/, $in{$_[0]})) {
	&check_ipaddress($addr) || &error(&text('eip', $addr));
	push(@vals, { 'name' => $addr });
	}
$dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
&save_directive($_[1], $_[0], @vals ? [ $dir ] : [ ], $_[2]);
}

# opt_input(text, name, &config, default, size, units)
sub opt_input {
  local($v, $rv, $n);
  $v = &find($_[1], $_[2]);
  ($n = $_[1]) =~ s/[^A-Za-z0-9_]/_/g;

  $rv  = "<td valign=top><DIV CLASS=nbody><b>$_[0]</b></td>\n";
  $rv .= "<td nowrap valign=top";
  $rv .= $_[4] > 30 ? " colspan=3>\n" : ">\n";
  $rv .= "<FONT FACE=Arial,Helvetica,Sans-Serif>\n";
  $rv .= sprintf "<input type=radio name=${n}_def value=1 %s> $_[3]\n",
	  $v ? "" : "checked";
  $rv .= sprintf "<input type=radio name=${n}_def value=0 %s></FONT> ",
	  $v ? "checked" : "";
  $rv .= sprintf "<input name=$n size=$_[4] value=\"%s\"> $_[5]</td>\n",
	  $v ? $v->{'value'} : "";

  return $rv;
}

sub save_opt
{
local($dir, $n);
($n = $_[0]) =~ s/[^A-Za-z0-9_]/_/g;
if ($in{"${n}_def"}) { &save_directive($_[2], $_[0], [ ], $_[3]); }
elsif ($err = &{$_[1]}($in{$n})) {
	&error($err);
	}
else {
	$dir = { 'name' => $_[0], 'values' => [ $in{$n} ] };
	&save_directive($_[2], $_[0], [ $dir ], $_[3]);
	}
}

# directives that need their value to be quoted
@need_quote = ( "file", "zone", "pid-file", "statistics-file",
	        "dump-file", "named-xfer" );
foreach $need (@need_quote) {
	$need_quote{$need}++;
	}

1;

# find_reverse(address)
# Returns the zone and record structures for the PTR record for some address
sub find_reverse
{
local($conf, @zl, $rev, $z, $revconf, $revfile, $revrec, @revrecs, $addr, $rr);

# find reverse domain
$conf = &get_config();
@zl = &find("zone", $conf);
@octs = split(/\./, $_[0]);
DOMAIN: for($i=2; $i>=0; $i--) {
	$rev = &ip_to_arpa(join('.', @octs[0..$i]));
	$rev =~ s/\.$//g;
	foreach $z (@zl) {
		if (lc($z->{'value'}) eq $rev &&
		    ($typed = &find("type", $z->{'members'})) &&
		    $typed->{'value'} eq "master") {
			# found the reverse master domain
			$revconf = $z;
			last DOMAIN;
			}
		}
	}

# find reverse record
if ($revconf) {
	$revfile = &find("file", $revconf->{'members'})->{'value'};
	@revrecs = &read_zone_file($revfile, $revconf->{'value'});
	$addr = &ip_to_arpa($_[0]);
	foreach $rr (@revrecs) {
		if ($rr->{'type'} eq "PTR" &&
		    lc($rr->{'name'}) eq lc($addr)) {
			# found the reverse record
			$revrec = $rr;
			last;
			}
		}
	}
return ($revconf, $revfile, $revrec);
}

# find_forward(address)
# Returns the zone and record structures for the A record for some address
sub find_forward
{
local ($fwdconf, $i, $fwdfile, $fwdrec, $fr);

# find forward domain
local $host = $_[0]; $host =~ s/\.$//;
local $conf = &get_config();
local @zl = &find("zone", $conf);
local @parts = split(/\./, $host);
DOMAIN: for($i=1; $i<@parts; $i++) {
	local $fwd = join(".", @parts[$i .. @parts-1]);
	foreach $z (@zl) {
		local $typed;
		if (lc($z->{'value'}) eq $fwd &&
		    ($typed = &find("type", $z->{'members'})) &&
		    $typed->{'value'} eq "master") {
			# Found the forward master!
			$fwdconf = $z;
			last DOMAIN;
			}
		}
	}

# find forward record
if ($fwdconf) {
	$fwdfile = &find("file", $fwdconf->{'members'})->{'value'};
	local @fwdrecs = &read_zone_file($fwdfile, $fwdconf->{'value'});
	foreach $fr (@fwdrecs) {
		if ($fr->{'type'} eq 'A' &&
		    $fr->{'name'} eq $_[0]) {
			# found the forward record!
			$fwdrec = $fr;
			last;
			}
		}
	}

return ($fwdconf, $fwdfile, $fwdrec);
}

# can_edit_zone(&access, zone)
sub can_edit_zone
{
local %zcan;
return 1 if ($access{'zones'} eq '*');
foreach (split(/\s+/, $access{'zones'})) {
	return 1 if ($_ eq $_[1]);
	}
return 0;
}

### ############################################################################
### record_input(zoneindex, type, file, origin, [num], [record])
### Display a form for editing or creating a DNS record
### ############################################################################
sub record_input {
  local(%rec, @recs);
  print "<form action=save_record.cgi>\n";
  print "<input type=hidden name=index value=$_[0]>\n";
  print "<input type=hidden name=file value=$_[2]>\n";
  print "<input type=hidden name=origin value=$_[3]>\n";

  if (@_ >= 5) {
    print "<input type=hidden name=num value=$_[4]>\n";
    %rec = %{$_[5]};
  }
  else { print "<input type=hidden name=new value=1>\n"; }


  print "<input type=hidden name=type value=$_[1]>\n";
  print "<CENTER><table border=0 WIDTH=100% CELLPADDING=0 CELLSPACING=0>\n";
  print "<tr $tb>\n";
  print "  <td><DIV CLASS=thead><b>",&text(@_ >= 5 ? 'edit_edit' : 'edit_add',
			      $text{"type_$_[1]"}),"</b></td> </tr>\n";

  print "<TR $cb>\n";

  ### Mail Exchanger.
  if ($_[1] eq "MX") {
    print "  <TD><DIV CLASS=nbody>Below you can define what machine you want ";
    print "to recieve e-mail for your domain.<P>Enter the machine name in the ";
    print "<B>Mail Server</B> field, and the priority in the <B>Priority</B> ";
    print "field.</TD>\n";

    print "</TR>\n";
    print "<TR $cb><TD>&nbsp;</TD></TR>\n";
    print "<tr $cb>\n";
    print "  <td><table width=100%>\n";

    #print "<tr> <td><DIV CLASS=nbody><B>",
      #&hlink("Domain Name", "domainname-mx"), 
      #"</B></td>\n";
    #print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
  }

  ### Canonical Name.
  elsif ($_[1] eq "CNAME") {
    print "  <TD><DIV CLASS=nbody>Below you can define aliases to different ";
    print "machines.  <B>Alias</B> is the machine you wish to alias to <B>Real ";
    print "Name</B>  <P>For example, you may want to alias ";
    print "www.guardiandigital.com to webserver1.guardiandigital.com.  In ";
    print "this case www.guardiandigital.com is your <B>Alias</B> and ";
    print "webserver1.guardiandigital.com is your <B>Real Name</B>\n";

    print "<P><B>Warning:</B> You can not make an alias to a mail server.\n";

    print "</TR>\n";
    print "<TR $cb><TD>&nbsp;</TD></TR>\n";
    print "<tr $cb>\n";
    print "  <td><table width=100%>\n";
    print "<tr> <td><DIV CLASS=nbody><B>",
      &hlink("Alias", "alias-cname"), 
      "</B></td>\n";
    print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
  }

  ### Name Server.
  elsif ($_[1] eq "NS") {
    print "  <TD><DIV CLASS=nbody>Below you can define namservers for your ";
    print "domain.  <P>Enter your domain in the <B>Domain Name</B> field and ";
    print "the name of the name server in the  <B>Name Server</B> field.";

    print "</TR>\n";
    print "<TR $cb><TD>&nbsp;</TD></TR>\n";
    print "<TR $cb>\n";
    print "  <TD><table width=100%>\n";
    #print "    <TR><TD><DIV CLASS=nbody><B>",
      #&hlink("Domain Name", "domainname-ns"), "</B></TD>\n";
    #print "        <TD><input name=name value=\"$rec{'name'}\" size=30></TD>\n";
  }

  ### Address Record.
  elsif ($_[1] eq "A") {
    print "  <TD><DIV CLASS=nbody>Below you can define an address record. ";
    print "<P>Enter the fully-qualified domain name in the <B>Hostname</B> ";
    print "field, and the IP address you wish to assign to it in the ";
    print "<B>Address</B> field.";

    print "</TR>\n";
    print "<TR $cb><TD>&nbsp;</TD></TR>\n";
    print "<tr $cb>\n";
    print "  <td><table width=100%>\n";
    print "<tr> <td><DIV CLASS=nbody><B>",
      &hlink("Hostname", "hostname-a"), 
      "</B></td>\n";
    print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
  }

  ### Reverse Pointer.
  elsif ($_[1] eq "PTR") {
    print "  <TD><DIV CLASS=nbody>Below you can define a reverse record. ";
    print "<P>Enter the IP address in the <B>Address</B> field, and the ";
    print "fully-qualified domain name in the <B>Hostname</B> field.\n";

    print "</TR>\n";
    print "<TR $cb><TD>&nbsp;</TD></TR>\n";
    print "<tr $cb>\n";
    print "  <td><table width=100%>\n";
    print "<tr> <td><DIV CLASS=nbody><b>";
    print &hlink($text{'edit_addr'}, "address");
    print "</b></td>\n";
    printf "<td><input name=name value=\"%s\" size=30></td>\n",
           !%rec && $_[3] =~ /^(\d+)\.(\d+)\.(\d+)\.in-addr/ ?
           "$3.$2.$1." : &arpa_to_ip($rec{'name'});
  }

  ### Unknown type.
  else {
    print "  <TD><DIV CLASS=nbody>&nbsp;</TD>\n";

    print "</TR>\n";
    print "<TR $cb><TD>&nbsp;</TD></TR>\n";
    print "<tr $cb>\n";
    print "  <td><table width=100%>\n";
    print "<tr> <td><DIV CLASS=nbody><B>",
      &hlink($text{'edit_name'}, "fqdn"), 
      "</B></td>\n";
    print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
  }

  print "<TD COLSPAN=2><INPUT TYPE=HIDDEN NAME=ttl_def VALUE=1></TD></TR>\n";

  @v = @{$rec{'values'}};

  if ($_[1] eq "A") {
    print "<tr> <td><DIV CLASS=nbody><b>";
    print &hlink($text{'value_A1'}, "address");
    print "</b></td>\n";
    print "<td><input name=value0 size=20 value=\"$v[0]\"></td> </tr>\n";

    if (@_ >= 5) {
      print "<input type=hidden name=oldname ", "value=\"$rec{'name'}\">\n";
      print "<input type=hidden name=oldvalue0 value=\"$v[0]\">\n";
    }
  }
  elsif ($_[1] eq "NS") {
    print "<tr>\n";
    print "  <TD><DIV CLASS=nbody><B>\n"; 
    print &hlink($text{'value_NS1'}, "fqdn-ns");
    print "  </B></TD>\n"; 

    print "  <td colspan=3>\n";
    print "    <input name=value0 size=30 value=\"$v[0]\">\n";
    print "    <input type=hidden name=name value=$_[3]>\n";
    print "  </td>\n";
    print "</tr>\n";
  }
  elsif ($_[1] eq "CNAME") {
    print "<tr>\n";

    print "  <TD><DIV CLASS=nbody><B>\n";
    print &hlink($text{'value_CNAME1'}, "fqdn-cname");
    print "  </B></TD>\n";

    print "<td colspan=3><input name=value0 size=30 value=\"$v[0]\">\n";
  }
  elsif ($_[1] eq "MX") {
    print "<tr>\n";

    print "  <td><DIV CLASS=nbody><b>\n";
    print &hlink($text{'value_MX2'}, "fqdn-mx");
    print "  </b></td>\n";

    print "<td><input name=value1 size=30 value=\"$v[1]\">\n";
    print "    <input type=hidden name=name value=$_[3]></TD>\n";
    print "<td><DIV CLASS=nbody><b>";
    print &hlink($text{'value_MX1'}, "priority");
    print "</b></td>\n";
    print "<td><input name=value0 size=3 value=\"$v[0]\" MAXLENGTH=3>";
    print "</TD></TR>\n\n";
  }
  elsif ($_[1] eq "PTR") {
    print "<tr> <td><DIV CLASS=nbody><b>";
    print &hlink($text{'value_PTR1'}, "hostname-a");
    print "</b></td>\n";
    print "<td><input name=value0 size=30 value=\"$v[0]\"></td> </tr>\n";

    if (@_ >= 5) {
      print "<input type=hidden name=oldname ", "value=\"$rec{'name'}\">\n";
      print "<input type=hidden name=oldvalue0 value=\"$v[0]\">\n";
    }
  }



  if ($_[1] eq "A") {
    print "<tr>\n";
    print "    <TD><INPUT TYPE=HIDDEN NAME=\"rev\" VALUE=1></TD>\n";
  }
  elsif ($_[1] eq "PTR") {
    print "<tr>\n";
    print "    <TD><INPUT TYPE=HIDDEN NAME=\"fwd\" VALUE=1></TD>\n";
    #print "  <td><DIV CLASS=nbody><b>";
    #print &hlink($text{'edit_upfwd'}, "upfwd");
    #print "</b></td>\n";

    #printf "<td><DIV CLASS=nbody><input type=radio name=fwd value=1 %s> $text{'yes'}\n",
           #$config{'rev_def'} ? '' : 'checked';
    #printf "<input type=radio name=fwd value=0 %s> $text{'no'}</td>\n",
           #$config{'rev_def'} ? 'checked' : '';
  }
  else { print "<tr> <td colspan=2></td>\n"; }



  print "<td colspan=2 align=right>\n";

  local %a = &get_module_acl();

  if (!$a{'ro'}) {
    if (@_ >= 5) {
      print "<DIV CLASS=nbody>\n";
      print "<input type=submit value=\"$text{'save'}\">\n";
      print "<input type=submit name=delete ", "value=\"$text{'delete'}\">\n";
    }
    else {
      print "<DIV CLASS=nbody>\n";
      print "<input type=submit value=\"$text{'create'}\">\n";
    }
  }

  print "</td></tr></table></td></tr></table></form></CENTER>\n";
}


################################################################################
sub zones_table {
  local($i);

  print "<table border=0 WIDTH=100% CELLPADDING=0 CELLSPACING=0>\n";
  print "<tr $tb>\n";
  print "  <td><DIV CLASS=thead><b>$text{'index_zone'}</b></td>\n";
  print "  <td><DIV CLASS=thead><b>$text{'index_type'}</b></td>\n";
  print "</tr>\n\n";

  for($i=0; $i<@{$_[0]}; $i++) {
    print "<tr $cb>\n";
    print "<td><DIV CLASS=nbody><a href=\"$_[0]->[$i]\">$_[1]->[$i]</a></td>\n";
    print "<td><DIV CLASS=nbody>$_[2]->[$i]</td>\n";
    print "</tr>\n";
  }

  print "</table>\n";
}

@cat_list = ( 'default', 'config', 'parser', 'queries',
	      'lame-servers', 'statistics', 'panic', 'update',
	      'ncache', 'xfer-in', 'xfer-out', 'db',
	      'eventlib', 'packet', 'notify', 'cname', 'security',
	      'os', 'insist', 'maintenance', 'load', 'response-checks' );

@syslog_levels = ( 'kern', 'user', 'mail', 'daemon', 'auth', 'syslog',
		   'lpr', 'news', 'uucp', 'cron', 'authpriv', 'ftp',
		   'local0', 'local1', 'local2', 'local3',
		   'local4', 'local5', 'local6', 'local7' );

@severities = ( 'critical', 'error', 'warning', 'notice', 'info',
		'debug', 'dynamic' );


################################################################################
sub ReloadNamed() {
  ### Clean and save the environment.
  my %ENV_RESTORE;
  %ENV_RESTORE  =  &clean_env();

    ### Restart the daemon.
    $out = `/etc/init.d/named reload`;

  ### Restore the environment.
  &restore_env(%ENV_RESTORE);
}

sub RestartNamed() {
  ### Clean and save the environment.
  my %ENV_RESTORE;
  %ENV_RESTORE  =  &clean_env();

    ### Restart the daemon.
    $out = `/etc/init.d/named restart`;

  ### Restore the environment.
  &restore_env(%ENV_RESTORE);
}


