#!/usr/bin/perl

  require './bind8-lib.pl';
  require "../../guardiandigital-lib.pl";
  &ReadParse();

  &error_setup($text{'mcreate_err'});
  %access = &get_module_acl();
  $access{'master'} || &error($text{'mcreate_ecannot'});
  $access{'ro'} && &error($text{'master_ero'});

  $conf = &get_config();


  ### Validate inputs.
  if ($in{'rev'}) {

    ### Make sure it's not a domain...
    if ($in{'zone'} !~ /^[0-9\.\-]+$/) {
      &PrintError("'$in{'zone'}' is not a valid network.");
      exit;
    }

    ### Make sure it's a valid network number.
    if ($in{'zone'} !~ m/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/) {
      &PrintError("$in{'zone'} is not a valid network address.");
      exit;
    }

    ### Lop off the trailing ".[0-9]{3}" for our in-addr.arpa record.
    else { $in{'zone'} =~ s/\.[0-9]$//; }

    $in{'zone'} = &ip_to_arpa($in{'zone'});
  }
  else {
    if ($in{'zone'} !~ /^[A-Za-z0-9\-\.]+$/) {
      &PrintError("'$in{'zone'}' is not a valid domain name.");
      exit;
    }
    elsif ($in{'zone'} =~ /^[0-9\.]+$/) {
      &PrintError("'$in{'zone'}' must be a domain, not a network.");
      exit;
    }
  }


  ### ########################## ###
  ### Check for duplicate zones. ###
  ### ########################## ###
  @zones = &find("zone", $conf);

  foreach $z (@zones) {
    if ($in{'zone'} =~ m/^$z->{'value'}\.?$/) {
      &PrintError("This zone already exists.");
      exit;
    }
  }
  ### ############################ ###
  ### /Check for duplicate zones./ ###
  ### ############################ ###

  ### Lop off the trailing dot.
  $in{'zone'} =~ s/\.$//g;

  ### Check the validity of the master server.
  if ($in{'master'} !~ /^[A-Za-z0-9\-\.]+$/) {
    &PrintError("'$in{'master'}' is not a valid master server.");
    exit;
  }

  if ($in{'master'} !~ /\.$/) { $in{'master'} .= "."; }

  ### Check the validity of the e-mail address.
  if ($in{'email'} !~ /^\S+$/) {
    &PrintError("'$in{'email'}' is not a valid e-mail address");
    exit;
  }

  ### Translate the '@' to a '.' for our SOA.
  if ($in{'email'} =~ s/\@/\./) { $in{'email'} .= "."; }

  ### Set the refresh, retry, expiry, and minimum defaults.
  $in{'refresh'} =~ /^\S+$/;
  $in{'retry'} =~ /^\S+$/;
  $in{'expiry'} =~ /^\S+$/;
  $in{'minimum'} =~ /^\S+$/;



  $base = $config{'master_dir'} ? $config{'master_dir'} :
          $access{'dir'} eq '/' ? &base_directory($conf) : $access{'dir'};

  if ($in{'tmpl'}) {
    for($i=0; $config{"tmpl_$i"}; $i++) {
      @c = split(/\s+/, $config{"tmpl_$i"}, 3);
      if ($c[1] eq 'A' && !&check_ipaddress($in{'ip'})) {
        &error($text{'master_eip'});
      }
    }
  }

  if (!$in{'file_def'}) {
    $in{'file'} =~ /^\S+$/ ||
       &error(&text('mcreate_efile', $in{'file'}));

    if ($in{'file'} !~ /^\//) { $in{'file'} = $base."/".$in{'file'}; }

    &allowed_zone_file(\%access, $in{'file'}) ||
       &error(&text('mcreate_efile2', $in{'file'}));
  }
  elsif ($in{'rev'}) {
    ### create filename for reverse zone
    ### $in{'file'} = $base."/".&arpa_to_ip($in{'zone'}).".rev";
    $in{'file'} = $base."/master/db.".&arpa_to_ip($in{'zone'});
  }
  else {
    ### create filename for forward zone
    ### $in{'file'} = $base."/".$in{'zone'}.".hosts";
    ### $in{'file'} = $base."/".$in{'zone'}.".soa";
    $in{'file'} = $base."/master/db.".$in{'zone'};
  }

  &lock_file("$config{'chroot'}$in{'file'}");
  open(ZONE, "> $config{'chroot'}$in{'file'}") ||
     &error(&text('mcreate_efile3', $in{'file'}, $!));
  close(ZONE);

  $in{'file'} =~ s/^$config{'chroot'}//;

  ### create the SOA and NS records
  if ($config{'soa_style'} == 1) { $serial = &date_serial()."00"; }
  else { $serial = time(); }

  $vals = "$in{'master'} $in{'email'} (\n".  "\t\t\t$serial\n".
          "\t\t\t$in{'refresh'}\n". "\t\t\t$in{'retry'}\n".
          "\t\t\t$in{'expiry'}\n". "\t\t\t$in{'minimum'} )";


  ### Define a default TTL of one day.
  open ZONE, ">$config{'chroot'}/$in{'file'}";
    print ZONE "\$TTL 1d\n";
  close ZONE;

  &create_record($in{'file'}, "$in{'zone'}.", undef, "IN", "SOA", $vals);
  &create_record($in{'file'}, "$in{'zone'}.", undef, "IN", "NS", $in{'master'})
    if ($in{'master_ns'});

  if ($in{'tmpl'}) {
    ### Create template records.
    for($i=0; $config{"tmpl_$i"}; $i++) {
      @c = split(/\s+/, $config{"tmpl_$i"}, 3);
      &create_record($in{'file'}, $c[0] eq '.' ? "$in{'zone'}." : $c[0], undef,
                     "IN", $c[1], $c[2] ? $c[2] : $in{'ip'});
    }
  }

  &unlock_file("$config{'chroot'}$in{'file'}");

  ### Create the zone directive.
  &lock_file($config{'chroot'}.$config{'named_conf'});

  open(CONF, ">>$config{'chroot'}$config{'named_conf'}");
    print CONF "zone \"$in{'zone'}\" {\n";
    print CONF "	type master;\n";
    print CONF "	file \"$in{'file'}\";\n";

    ### allow-transfer sub-statement.
    if ($in{'allow-transfer_def'} == 0) {
      @transfers = split /\s+/, $in{'allow-transfer'}; 
      $i = 0;

      foreach $t (@transfers) { 
        if ($i == 0) { print CONF "	allow-transfer { $t; "; }
        else { print CONF "$t; "; }
        $i++; 
      }

      print CONF "};\n";
    }
    else { print CONF "	allow-transfer { none; };\n"; }

    ### allow-query sub-statement.
    if ($in{'allow-query_def'} == 0) {
      @queries = split /\s+/, $in{'allow-query'}; 
      $i = 0;

      foreach $q (@queries) { 
        if ($i == 0) { print CONF "	allow-query { $q; "; }
        else { print CONF "$q; "; }
        $i++; 
      }

      print CONF "};\n";
    }
    else { print CONF "	allow-query { any; };\n"; }

    print CONF "};\n\n";
    print CONF "\n";
  close(CONF);
  &unlock_file($config{'chroot'}.$config{'named_conf'});

  ### Add the new zone to the access list
  ### if ($access{'zones'} ne '*') {
    ### $access{'zones'} .= " ".$in{'zone'};
    ### &save_module_acl(\%access);
  ### }


  &RestartNamed();

  &redirect("");

