# redhat-linux-lib.pl
# Networking functions for redhat linux

$net_scripts_dir = "/etc/sysconfig/network-scripts";
$network_config = "/etc/sysconfig/network";
$static_route_config = "/etc/sysconfig/static-routes";

# active_interfaces()
# Returns a list of currently ifconfig'd interfaces
sub active_interfaces {
  local(@rv, @lines, $l);

  open(IFC, "ifconfig -a |");
    while(<IFC>) {
      s/\r|\n//g;
      if (/^\S+/) { push(@lines, $_); }
      else { $lines[$#lines] .= $_; }
    }
  close(IFC);

  foreach $l (@lines) {
    local %ifc;
    $l =~ /^([^:\s]+)/; $ifc{'name'} = $1;
    $l =~ /^(\S+)/; $ifc{'fullname'} = $1;

    if ($l =~ /^(\S+):(\d+)/) { $ifc{'virtual'} = $2; }

    if ($l =~ /inet addr:(\S+)/) { $ifc{'address'} = $1; }
    elsif (!$_[0]) { next; }

    if ($l =~ /Mask:(\S+)/) { $ifc{'netmask'} = $1; }
    if ($l =~ /Bcast:(\S+)/) { $ifc{'broadcast'} = $1; }
    if ($l =~ /HWaddr (\S+)/) { $ifc{'ether'} = $1; }
    if ($l =~ /MTU:(\d+)/) { $ifc{'mtu'} = $1; }

    $ifc{'up'}++ if ($l =~ /\sUP\s/);
    $ifc{'edit'} = ($ifc{'name'} !~ /^ppp/);
    $ifc{'index'} = scalar(@rv);

    push(@rv, \%ifc);
  }

  return @rv;
}

### activate_interface(&details)
### Create or modify an interface
sub activate_interface {
  local $a = $_[0];
  local $cmd = "ifconfig $a->{'name'}";

  ### if ($a->{'virtual'} ne "") {
    ### $cmd .= ":$a->{'virtual'}";
  ### }

  $cmd .= " $a->{'address'}";

  if ($a->{'netmask'}) { $cmd .= " netmask $a->{'netmask'}"; }
  if ($a->{'broadcast'}) { $cmd .= " broadcast $a->{'broadcast'}"; }
  if ($a->{'mtu'}) { $cmd .= " mtu $a->{'mtu'}"; }

  if ($a->{'up'}) { $cmd .= " up"; }
  else { $cmd .= " down"; }

  local $out = `$cmd 2>&1`;

  ### if ($?) { &error($out); }

  if ($a->{'ether'}) {
    $out = `ifconfig $a->{'name'} hw ether $a->{'ether'} 2>&1`;
    ### if ($?) { &error($out); }
  }
}

### deactivate_interface(&details)
### Shutdown some active interface
sub deactivate_interface {
  local $name = $_[0]->{'name'}.
                ($_[0]->{'virtual'} ne "" ? ":$_[0]->{'virtual'}" : "");

  if ($_[0]->{'virtual'} ne "") {
    ### Shutdown virtual interface by setting address to 0
    local $out = `ifconfig $name 0 2>&1`;
    if ($?) { &error($out); }
  }

local ($still) = grep { $_->{'fullname'} eq $name } &active_interfaces();
if ($still) {
	# Old version of ifconfig or non-virtual interface.. down it
	local $out = `ifconfig $name down 2>&1`;
	if ($?) { &error($out); }
	}
}

### ############################################################################
### boot_interfaces()
### Returns a list of interfaces brought up at boot time.
### ############################################################################
sub boot_interfaces {
  local(@rv, $f);

  open CONF, "/etc/network/interfaces";

  $i = 0; $line = 0;
  while (<CONF>) {
    chomp $_;

    ### Incerement our line number.
    $line++;

    ### Skip comments.
    if ($_ =~ m/^\#/) { next; }


    ### New line, let's start populating our temp hash.
    elsif ($_ =~ m/^iface\ (.*)\ (.*)\ (.*)$/) {
      ### Save this line as where the previous interface ends.
      if ($interface[$i]->{'name'}) { $interface[$i++]->{'end'} = $line; }

      ### Begin new interface.
      $interface[$i]->{'index'}    = $i;
      $interface[$i]->{'name'}     = $1;
      $interface[$i]->{'fullname'} = $1;
      $interface[$i]->{'service'}  = $2;
      $interface[$i]->{'type'}     = $3;
      $interface[$i]->{'line'}     = $line;

      if ($interface[$i]->{'name'} =~ m/^eth[0-9]+\:[0-9]+$/) {
        $interface[$i]->{'virtual'}     = "1";
      }

    }

    ### This interface is the default gateway.
    elsif ($_ =~ m/default\ gw\ ([0-9\.]+)/) {
      $interface[$i]->{'defgw'} = $1;
      $interface[$i]->{'defgwline'} = $line;
    }

    ### We have a key = value line.
    elsif ($_ =~ m/([a-z]+)\ ([0-9\.]+)$/)   { $interface[$i]->{$1} = $2; }
  }

  close CONF;

  return @interface;
}


### ############################################################################
### get_routes()
### Returns a list of static routes.
### ############################################################################
sub get_routes() {

  open CONF, "/etc/network/interfaces";

  $i = 0; $line = 0;
  while (<CONF>) {
    chomp $_;

    ### Incerement our line number.
    $line++;

    ### Skip comments.
    if ($_ =~ m/^\#/) { next; }

    ### New line, let's start populating our temp hash.
    ### We have a route line.
    if ($_ =~ m/^\s+up\ (.*)$/) {
      $my_line = $1;

      if ($my_line !~ m/default\ gw/) {
        $tmp[$i]->{'line'} = $line;

        if ($my_line =~ m/\-net\ ([^\ ]+)/)   { $tmp[$i]->{'net'} = $1; }
        if ($my_line =~ m/dev\ ([^\ ]+)/)     { $tmp[$i]->{'dev'} = $1; }
        if ($my_line =~ m/netmask\ ([^\ ]+)/) { $tmp[$i]->{'mask'} = $1; }
        if ($my_line =~ m/gw\ ([^\ ]+)/)      { $tmp[$i]->{'gw'} = $1; }

        $i++;
      }
    }

  }

  close CONF;

  return @tmp;
}

### ############################################################################
### save_interface(&details)
### Create or update a boot-time interface
### ############################################################################
sub save_interface {
  local (%conf);
  (%conf) = @_;
  local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'} : $_[0]->{'name'};

  ### ###################################
  ### Pull our new values into variables.
  ### ###################################
  $new_index     = $_[0]->{'index'};
  $new_lineno    = $_[0]->{'line'};
  $new_name      = $_[0]->{'name'};
  $new_address   = $_[0]->{'address'};
  $new_netmask   = $_[0]->{'netmask'};
  $new_broadcast = $_[0]->{'broadcast'};

  ### Calculate the network number
  $tmp = `ipcalc --network $new_address $new_netmask`;
  ($tmp, $new_network) = split /\=/, $tmp; chomp $new_network;



  ### ###################################################################### ###
  ### We are simply adding an interface so let's just append it to the file.
  ### ###################################################################### ###
  if ($_[0]->{'new'}) {
    $lref = &read_file_lines("/etc/network/interfaces");
      push @$lref, "";
      push @$lref, "iface $new_name inet static";
      push @$lref, "   address $new_address";
      push @$lref, "   netmask $new_netmask";
      push @$lref, "   network $new_network";
      push @$lref, "   broadcast $new_broadcast";
      push @$lref, "";
    &flush_file_lines("/etc/network/interfaces");
  }

  ### ######################################## ###
  ### We want to modify an existing interface. ###
  ### ######################################## ###
  else {
    ### Read in our current interfaces.
    @interfaces = &boot_interfaces();

    ### Read the file in.
    $lref = &read_file_lines("/etc/network/interfaces");

      ### Get the range we wish to edit...
      $start = $interfaces[$new_index + 1]->{'line'} - 1;
      $end   = $interfaces[$new_index + 1]->{'end'} - 1;
      if ($end < 0) { $end = @$lref; }

      ### Calculate the network number
      $tmp = `ipcalc --network $new_address $new_netmask`;
      ($tmp, $new_network) = split /\=/, $tmp; chomp $new_network;

      ### Edit the values in our range, keeping routes intact.
      for ($j = $start; $j < $end; $j++) { 
        if (@$lref[$j] =~ m/^\s+address/) {
          @$lref[$j] = "   address $new_address";
        }
        elsif (@$lref[$j] =~ m/^\s+netmask/) {
          @$lref[$j] = "   netmask $new_netmask";
        }
        elsif (@$lref[$j] =~ m/^\s+broadcast/) {
          @$lref[$j] = "   broadcast $new_broadcast";
        }
        elsif (@$lref[$j] =~ m/^\s+network/) {
          @$lref[$j] = "   network $new_network";
        }
      }

  ### Write the file back to disk.
  &flush_file_lines("/etc/network/interfaces");

  }
}



### ############################################################################
### delete_interface(&details)
### Delete a boot-time interface
### ############################################################################
sub delete_interface {
  local ($index);
  ($index) = @_;

  ### Read in our current interfaces.
  @interfaces = &boot_interfaces();

  ### Save a backup copy.
  system "cp -a /etc/network/interfaces /etc/network/interfaces.wtsave";

  $lref = &read_file_lines("/etc/network/interfaces");

    ### Get the start and end lines for this interface.
    $start = $interfaces[$index + 1]->{'line'};
    $end   = $interfaces[$index + 1]->{'end'};

    ### Calculate the length to splice.
    $length = $end - $start;

    if ($end) { splice @$lref, $interfaces[$index + 1]->{'line'} - 1, $length; }
    else      { splice @$lref, $interfaces[$index + 1]->{'line'} - 1, 100; }

  &flush_file_lines();


}



### iface_type(name)
### Returns a human-readable interface type name
sub iface_type {
  return "PPP" if ($_[0] =~ /^ppp/);
  return "SLIP" if ($_[0] =~ /^sl/);
  return "PLIP" if ($_[0] =~ /^plip/);
  return "Ethernet" if ($_[0] =~ /^eth/);
  return "Arcnet" if ($_[0] =~ /^arc/);
  return "Token Ring" if ($_[0] =~ /^tr/);
  return "Pocket/ATP" if ($_[0] =~ /^atp/);
  return "Loopback" if ($_[0] =~ /^lo/);
  return "Ethernet" if ($_[0] =~ /^static/);
  return "Unknown";
}



### iface_hardware(name)
### Does some interface have an editable hardware address
sub iface_hardware { return $_[0] =~ /^eth/; }

#### can_edit(what)
#### Can some boot-time interface parameter be edited?
sub can_edit { return $_[0] ne "mtu"; }

### valid_boot_address(address)
### Is some address valid for a bootup interface
sub valid_boot_address { return &check_ipaddress($_[0]); }

### get_dns_config()
### Returns a hashtable containing keys nameserver, domain, search & order
sub get_dns_config
{
local $dns;
open(RESOLV, "/etc/resolv.conf");
while(<RESOLV>) {
	s/\r|\n//g;
	if (/nameserver\s+(.*)/) {
		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
		}
	elsif (/domain\s+(\S+)/) {
		$dns->{'domain'} = [ $1 ];
		}
	elsif (/search\s+(.*)/) {
		$dns->{'domain'} = [ split(/\s+/, $1) ];
		}
	}
close(RESOLV);
open(SWITCH, "/etc/nsswitch.conf");
while(<SWITCH>) {
	s/\r|\n//g;
	if (/^\s*hosts:\s+(.*)/) {
		$dns->{'order'} = $1;
		}
	}
close(SWITCH);
return $dns;
}

### ############################################################################
### save_dns_config(&config)
### Writes out the resolv.conf and nsswitch.conf files
### ############################################################################
sub save_dns_config {

  ### Read the old resolv.conf.
  open(RESOLV, "/etc/resolv.conf");
    local @resolv = <RESOLV>;
  close(RESOLV);

  ### Write the new one back.
  open(RESOLV, ">/etc/resolv.conf");

    ### Nameserver's up top.
    foreach (@{$_[0]->{'nameserver'}}) { print RESOLV "nameserver $_\n"; }

    if ($_[0]->{'domain'}) {
      if ($_[0]->{'domain'}->[1]) {
        print RESOLV "search ",join(" ", @{$_[0]->{'domain'}}),"\n";
      }
      else { print RESOLV "domain $_[0]->{'domain'}->[0]\n"; }
    }

    foreach (@resolv) {
      print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
    }
  close(RESOLV);

  `cp -f /etc/resolv.conf /var/spool/postfix/etc/resolv.conf`

}

$max_dns_servers = 3;

# order_input(&dns)
# Returns HTML for selecting the name resolution order
sub order_input
{
if ($_[0]->{'order'} =~ /\[/) {
	# Using a complex resolve list
	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
	}
else {
	# Can select by menus
	local @o = split(/\s+/, $_[0]->{'order'});
	@o = map { s/nis\+/nisplus/; s/yp/nis/; $_; } @o;
	local ($rv, $i, $j);
	#local @srcs = ( "", "files", "dns", "nis", "nisplus", "db" );
	#local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+", "DB" );
	local @srcs = ( "", "files", "dns" );
	local @srcn = ( "", "Hosts", "DNS" );
	for($i=1; $i<@srcs; $i++) {
		local $ii = $i-1;
		$rv .= "<select name=order_$ii>\n";
		for($j=0; $j<@srcs; $j++) {
			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
					$srcs[$j],
					$o[$ii] eq $srcs[$j] ? "selected" : "",
					$srcn[$j];
			}
		$rv .= "</select>\n";
		}
	return $rv;
	}
}

# parse_order(&dns)
# Parses the form created by order_input()
sub parse_order
{
if (defined($in{'order'})) {
	$in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
	$_[0]->{'order'} = $in{'order'};
	}
else {
	local($i, @order);
	for($i=0; defined($in{"order_$i"}); $i++) {
		push(@order, $in{"order_$i"}) if ($in{"order_$i"});
		}
	$_[0]->{'order'} = join(" ", @order);
	}
}

# get_hostname()
sub get_hostname
{
return &get_system_hostname();
}

# save_hostname(name)
sub save_hostname
{
local %conf;
system("hostname $_[0] >/dev/null 2>&1");
open(HOST, ">/etc/HOSTNAME");
print HOST $_[0],"\n";
close(HOST);
&read_env_file($network_config, \%conf);
$conf{'HOSTNAME'} = $_[0];
&write_env_file($network_config, \%conf);
}

# get_domainname()
sub get_domainname
{
local $d = `domainname`;
chop($d);
return $d eq "(none)" ? "" : $d;
}

# save_domainname(domain)
sub save_domainname
{
local %conf;
system("domainname \"$_[0]\" >/dev/null 2>&1");
&read_env_file($network_config, \%conf);
if ($_[0]) {
	$conf{'NISDOMAIN'} = $_[0];
	}
else {
	delete($conf{'NISDOMAIN'});
	}
&write_env_file($network_config, \%conf);
}

### ################ ###########################################################
### &routing_input() ###########################################################
### ################ ###########################################################
sub routing_input {
  ### Show default router and device.
  local (%conf, @st, @hr);

  foreach $b (&boot_interfaces()) {
    if ($b->{'defgw'}) {
      $gateway  = $b->{'defgw'};
      $gatedev  = $b->{'name'};
      $gateline = $b->{'defgwline'};
    }
  } 

  print "<CENTER><TABLE BORDER=0 WIDTH=620 CELLPADDING=0 CELLSPACING=0>\n";
  print "<TR $cb>\n";
  print "  <TD WIDTH=20>&nbsp;&nbsp;</TD>\n";
  print "  <TD NOWRAP>\n";
  print "    <FORM ACTION=save_gw.cgi METHOD=POST>\n";
  print "    <DIV CLASS=nbody><b>$text{'routes_default'}</b>\n";
  print "  </TD>\n";
  print "  <TD><input name=gateway size=15 MAXLENGTH=15 value=$gateway></TD>\n";
  print "  <TD NOWRAP><DIV CLASS=nbody><B>Routing Device</B></TD>\n";
  print "  <TD WIDTH=80%><DIV CLASS=nbody>\n";

  foreach $b (&boot_interfaces()) {
    if ($b->{'name'} ne "lo") {
      if ($b->{'defgw'}) {
        print "    <INPUT TYPE=RADIO NAME=gatedev VALUE=$b->{'name'} CHECKED> $b->{'name'}\n";
      }
      elsif (!$b->{'virtual'}) {
        print "    <INPUT TYPE=RADIO NAME=gatedev VALUE=$b->{'name'}> $b->{'name'}\n";
      }
    }
  }

  print "    <INPUT TYPE=HIDDEN NAME=defgwline VALUE=$gateline>\n";
  print "  </TD>\n";
  print "  <TD><DIV CLASS=nbody>\n";
  print "    <INPUT TYPE=SUBMIT VALUE=Save>\n";
  print "    <P></FORM>\n";
  print "  </TD>\n";
  print "</TR>\n\n";

  print "</TABLE></CENTER>\n";

}

################################################################################
sub parse_routing {
  local ($device, $gateway);
  ($device, $gateway) = @_;
  
  ### Read in our current interfaces.
  @interfaces = &boot_interfaces();

  ### Save a backup copy.
  system "cp -a /etc/network/interfaces /etc/network/interfaces.wtsave";

  ### Get the lock and open the file for writing.
  &obtain_lock("/etc/network/interfaces");
  open CONF, ">/etc/network/interfaces";


  for ($j = 0; $j < @interfaces; $j++) {
    ### Localhost, print standard line.
    if ($interfaces[$j]->{'name'} eq "lo") {
      print CONF "iface lo inet loopback\n\n";
    }

    ### Another interface... let's re-print it.
    else {
      print CONF "iface $interfaces[$j]->{'name'} inet static\n";
      print CONF "   address $interfaces[$j]->{'address'}\n";
      print CONF "   netmask $interfaces[$j]->{'netmask'}\n";
      print CONF "   network $interfaces[$j]->{'network'}\n";
      print CONF "   broadcast $interfaces[$j]->{'broadcast'}\n";

      #if ($interfaces[$j]->{'gateway'}) {
      if ($interfaces[$j]->{'name'} eq $device) {
        print CONF "   gateway $gateway\n";
      }
      print CONF "\n";
    }
  }

  ### Close the file and release the lock.
  close CONF;
  &release_lock("/etc/network/interfaces");
}


################################################################################
sub add_route() {
  local ($device, $network, $netmask, $gateway);
  ($device, $network, $netmask, $gateway) = @_;


  ### Read the file in.
  open INTERFACES, "/etc/network/interfaces";
    while (<INTERFACES>) { chomp $_; push @interfaces, $_; }
  close INTERFACES;

  ### Build our new route string.
  if ($gateway) {
    $insert  = "   up route add -net $network netmask $netmask gw $gateway ";
    $insert .= "dev $device";
  }
  else {
    $insert = "   up route add -net $network netmask $netmask dev $device";
  }

  ### Insert the new route string.
  &obtain_lock("/etc/network/interfaces");
  open INTERFACES, ">/etc/network/interfaces";
    $in_block = 0;
    for ($i = 0; $i < @interfaces; $i++) {
      if ($interfaces[$i] =~ m/^iface $device\ /) { $in_block = 1; }
      if (($in_block) && ($interfaces[$i] eq "")) {
        $in_block = 0;
        print INTERFACES "$insert\n";
      }
      print INTERFACES "$interfaces[$i]\n";
    }
  close INTERFACES;
  &release_lock("/etc/network/interfaces");

}


################################################################################
sub save_default_gateway {
  local ($gateway, $gatewaydev, $gwline);
  ($gateway, $gatewaydev, $gwline) = @_;


  ### Remove the old gateway.
  $lref = &read_file_lines("/etc/network/interfaces");
  splice @$lref, $gwline - 1, 1;
  &flush_file_lines();


  ### Read the file back in.
  open INTERFACES, "/etc/network/interfaces";
    while (<INTERFACES>) { chomp $_; push @file, $_; }
  close INTERFACES;


  ### Build hash of start/end lines for each interface hash.
  $line = 0;
  foreach $f (@file) {
    if ($f =~ m/^iface ([^\ ]+)\ /) { $iface{$1}{'start'} = $line; }
    elsif ($f eq "")                { $iface{$1}{'end'} = $line; }
    $line++;
  }

  ### Add the new gateway.
  $insert = "   up route add default gw $gateway\n";


  $lref = &read_file_lines("/etc/network/interfaces");

  $i = 0;
  foreach (@$lref) { $i++; }

#print "Content-type: text/html\n\n<PRE>\n\n";

  if ($iface{$gatewaydev}{'end'}) { $END = $iface{$gatewaydev}{'end'}; }
  else                            { $END = $i; }

#print "SPLICING $END\n";

  #splice @$lref, $iface{$gatewaydev}{'end'}, 1, $insert;
  splice @$lref, $END, 1, $insert;
  &flush_file_lines();

#exit;
}

  ### ###############################################################
  ### PrintError()
  ### ###############################################################
  ### Used to pop up a JavaScript with a $message then return user to
  ### previous page.
  ### ###############################################################

  sub PrintError {
    my ($message) = @_;

    print <<ErrorEOF;
Content-type: text/html


<HTML>
<HEAD>
  <TITLE> </TITLE>

  <SCRIPT LANGUAGE="JavaScript">
    alert("$message");
    history.go(-1);
  </SCRIPT>
</HEAD>
</HTML>
ErrorEOF

  exit;
  }

################################################################################
### GetNextVirtualInterfaceID ##################################################
### Returns the ID of the next available virtual interface. ####################
################################################################################
sub GetNextVirtualInterfaceID() {
  my @boot = &boot_interfaces();
  my $next = 0;

  foreach $b (@boot) {
    if ($b->{'virtual'}) {
      my ($iface, $virt) = split /\:/, $b->{'name'};
      if ($virt > $next) { $next = $virt; }
    }
  }

  $next++;

  return $next;
}



1;

