# postfix-lib.pl

  do '/usr/webtool/web-lib.pl';
  &init_config();
  %access = &get_module_acl();

  $POSTFIX_MODULE_VERSION = 5;

### ############################################################################
### get_config_dir() 
### ############################################################################
sub guess_config_dir {
  my $answ = $config{'postfix_config_file'};
  $answ =~ /(.*)\/[^\/]*/;

  return $1;
}

$config_dir = guess_config_dir();



### ############################################################################
### postfix_module_version()
### returns the version of the postfix module
### ############################################################################
sub postfix_module_version { return $POSTFIX_MODULE_VERSION; }


### ############################################################################
### is_postfix_running()
### returns 1 if running, 0 if stopped, calls error() if problem
### ############################################################################
sub is_postfix_running {
  my $queuedir = get_current_value("queue_directory");
  my $processid = get_current_value("process_id_directory");
  my $pid_file = $queuedir."/".$processid."/master.pid";

  if (open(PID, $pid_file)) {
    chop(my $pid = <PID>);
    close(PID);
    $pid =~ /([0-9]+)/;

    return kill 0, $1;
  }
  else { return 0; }
}



sub is_existing_parameter {
  my $out = `$config{'postfix_config_command'} -c $config_dir $_[0] 2>&1`;
  return !($out =~ /unknown parameter/);
}


# get_current_value(parameter_name)
# returns a scalar corresponding to the value of the parameter
sub get_current_value
{
    my $out = `$config{'postfix_config_command'} -c $config_dir -h $_[0] 2>&1`;  # -h tells postconf not to output the name of the parameter
    if ($?) { &error("Failed to query Postfix config command to get the current value of parameter $_[0]: <tt>$out</tt>"); }
    chop($out);
    return $out;
}

# if_default_value(parameter_name)
# returns if the value is the default value
sub if_default_value
{
    my $out = `$config{'postfix_config_command'} -c $config_dir -n $_[0] 2>&1`;
    if ($?) { &error("Failed to query Postfix config command to get the current value of parameter $_[0]: <tt>$out</tt>"); }
    return ($out eq "");
}

# get_default_value(parameter_name)
# returns the default value of the parameter
sub get_default_value
{
    my $out = `$config{'postfix_config_command'} -c $config_dir -dh $_[0] 2>&1`;  # -h tells postconf not to output the name of the parameter
    if ($?) { &error("Failed to query Postfix config command to get the current value of parameter $_[0]: <tt>$out</tt>"); }
    chop($out);
    return $out;
}


# set_current_value(parameter_name, parameter_value)
# 
sub set_current_value
{
    my $value = $_[1];
#    print "--".$value."--<br>";
    if (($value eq "__DEFAULT_VALUE_IE_NOT_IN_CONFIG_FILE__") || ($value eq &get_default_value($_[0])))
    {
	# there is a special case in which there is no static default value ;
	# postfix will handle it correctly if I remove the line in `main.cf'
	my $all_lines = &read_file_lines($config{'postfix_config_file'});
	my $line_of_parameter = -1;
	my $i = 0;

	foreach (@{$all_lines})
	{
	    if (/^\s*$_[0]\s*=/)
	    {
		$line_of_parameter = $i;
	    }
	    $i++;
	}

	if ($line_of_parameter != -1)
	{
	    splice(@{$all_lines}, $line_of_parameter, 1);
	    
	    &flush_file_lines();
	}
    }
    else
    {
	$value =~ s/\$/\\\$/g;     # prepend a \ in front of every $ to protect from shell substitution
	$out = system("$config{'postfix_config_command'} -c $config_dir -e $_[0]=\"$value\" 2>&1");
    }
    if ($out) { &error("Failed to query Postfix config command to set the current value of parameter $_[0] to $_[1]: <tt>$out</tt><br> $config{'postfix_config_command'} -c $config_dir -e $_[0]=\"$value\" 2>&1"); }
}

# check_postfix()
#
sub check_postfix
{
	my $out = system("$config{'postfix_control_command'} -c $config_dir check 2>&1");
	return $out;
}

# reload_postfix()
#
sub reload_postfix
{
    $access{'startstop'} || &error("You are not allowed to reload Postfix");
    if (is_postfix_running())
    {
	### if (check_postfix()) { &error("$text{'check_error'}"); }
	my $out = system("$config{'postfix_control_command'} -c $config_dir reload 2>&1");
	if ($out) { &error("Failed to reload Postfix."); }
    }
}





### WARNING: *FIRST* RADIO BUTTON *MUST* BE THE DEFAULT VALUE OF POSTFIX
sub option_radios_freefield {
    my ($name, $length) = ($_[0], $_[1]);

    my $v = &get_current_value($name);
    my $key = 'opts_'.$name;

    my $check_free_field = 1;
    
    print "<TR><TD VALIGN=TOP><DIV CLASS=nbody><B>", 
          &hlink("<b>$text{$key}</B>", "opt_$name"), "</TD>\n";
    print "<TD>\n";

    ### first radio button (must be default value!!)
    printf "<DIV CLASS=toolfont><input type=radio name=$name"."_def value=\"__DEFAULT_VALUE_IE_NOT_IN_CONFIG_FILE__\" %s> $_[2]</DIV>\n", (&if_default_value($name)) ? "checked" : "";


    $check_free_field = 0 if &if_default_value($name); shift;
    
    ### other radio buttons
    while (defined($_[2])) {
      printf "<input type=radio name=$name"."_def value=\"$_[2]\" %s> $_[3]\n",
      ($v eq $_[2]) ? "checked" : "";
      if ($v eq $_[2]) { $check_free_field = 0; }
      shift; shift;
    }

    ### the free field
    printf "<BR><input type=radio name=$name"."_def value=__USE_FREE_FIELD__ %s>\n",
    ($check_free_field == 1) ? "checked" : "";
    printf "<input name=$name size=$length value=\"%s\"> </td>\n",
    ($check_free_field == 1) ? $v : "";

print "</TD></TR>\n\n";
}


sub option_freefield {
  my ($name, $length) = ($_[0], $_[1]);

  my $v = &get_current_value($name);
  my $key = 'opts_'.$name;
    
  print "<TR><TD VALIGN=TOP><B>", &hlink("<b>$text{$key}</B>", "opt_$name"), 
        "</TD>\n";

  print "<TD><input name=$name"."_def size=$length value=\"$v\"> </TD></TR>\n";
}


# option_yesno(name_of_option, [help])
# if help is provided, displays help link
sub option_yesno {
    my $name = $_[0];
    my $v = &get_current_value($name);
    my $key = 'opts_'.$name;

    defined($_[1]) ?
	print "<td>".&hlink("<b>$text{$key}</b>", "opt_".$name)."</td> <td nowrap>\n"
    :
	print "<td><b>$text{$key}</b></td> <td nowrap>\n";
    
    printf "<input type=radio name=$name"."_def value=\"yes\" %s> Yes\n",
    (lc($v) eq "yes") ? "checked" : "";

    printf "<input type=radio name=$name"."_def value=\"no\" %s> No\n",
    (lc($v) eq "no") ? "checked" : "";

    print "</td>\n";
}



############################################################################
# aliases support    [too lazy to create a aliases-lib.pl :-)]

# get_aliases_files($alias_maps) : @aliases_files
# parses its argument to extract the filenames of the aliases files
# supports multiple alias-files

sub get_aliases_files {
    $_[0] =~ /:(\/[^,\s]*)(.*)/;
    (my $returnvalue, my $recurse) = ( $1, $2 );

    return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         &get_aliases_files($recurse)
	     :
	         ()
           )
}

 
# get_aliases() : \@aliases
# construct the aliases database taken from the aliases files given in the 
# "alias_maps" parameter

sub get_aliases {
  if (!@aliases_cache) {
    my @aliases_files = &get_aliases_files(&get_current_value("alias_maps"));
    my $number = 0;

    foreach $aliases_file (@aliases_files) {
      open(ALIASES, $aliases_file);
      my $i = 0;

      while (<ALIASES>) {
        s/^#.*$//g;	# remove comments
        s/\r|\n//g;	# remove newlines

        if ((/^\s*\"([^\"]*)[^:]*:\s*([^#]*)/) || 
            (/^\s*([^\s:]*)[^:]*:\s*([^#]*)/)) {
          $number++;
          my %alias;
          $alias{'name'} = $1;
          $alias{'value'} = $2;
          $alias{'line'} = $i;
          $alias{'alias_file'} = $aliases_file;
          $alias{'number'} = $number;

          $alias{'comment'} = $4;

          push(@aliases_cache, \%alias);
        }

        $i++;
      }

      close(ALIASES);
    }
  }

  return \@aliases_cache;
}


# init_new_alias() : $number
# gives a new number of alias
sub init_new_alias
{
    $aliases = &get_aliases();

    my $max_number = 0;

    foreach $trans (@{$aliases})
    {
	if ($trans->{'number'} > $max_number) { $max_number = $trans->{'number'}; }
    }
    
    return $max_number+1;
}



# save_options(%options)
#
sub save_options
{
    ### if (check_postfix()) { &error("$text{'check_error'}"); }

    my %options = %{$_[0]};

    foreach $key (keys %options)
    {
	if ($key =~ /_def/)
	{
	    (my $param = $key) =~ s/_def//;
	    
	    ($options{$key} eq "__USE_FREE_FIELD__") ?
		&set_current_value($param, $options{$param})
		    :
		&set_current_value($param, $options{$key});
	}
    }
}


# regenerate_aliases
#
sub regenerate_aliases
{
    $access{'genaliases'} || error("You are not allowed to regenerate the Aliases");
    if (get_current_value("alias_maps") eq "")
    {
	$out = system("$config{'postfix_newaliases_command'} 2>&1");
	if ($out) { &error("Failed to regenerate aliases: <tt>$out</tt>"); }
    }
    else
    {
	foreach $map (get_maps_files(get_current_value("alias_maps")))
	{
	    $out = system("$config{'postfix_aliases_table_command'} -c $config_dir $map 2>&1");
	    if ($out) { &error("Failed to regenerate table $map: <tt>$out</tt>"); }
	}
    }
}


# regenerate_relocated_table
#
sub regenerate_relocated_table
{
    &regenerate_any_table("relocated_maps");
}


# regenerate_virtual_table
#
sub regenerate_virtual_table
{
    &regenerate_any_table("virtual_maps");
}


# regenerate_canonical_table
#
sub regenerate_canonical_table
{
    &regenerate_any_table("canonical_maps");
    &regenerate_any_table("recipient_canonical_maps");
    &regenerate_any_table("sender_canonical_maps");
}


### regenerate_transport_table
sub regenerate_transport_table { &regenerate_any_table("transport_maps"); }


### regenerate_any_table($parameter_where_to_find_the_table_names)
sub regenerate_any_table {
  if (&get_current_value($_[0]) ne "") {
    foreach $map (&get_maps_files(&get_current_value($_[0]))) {
      $out = system("$config{'postfix_lookup_table_command'} -c $config_dir  $map 2>&1");
      if ($out) { &error("Failed to regenerate table $map: <tt>$out</tt>"); }
    }
  }
}



############################################################################
# maps [canonical, virtual, transport] support

# get_maps_files($maps_param) : @maps_files
# parses its argument to extract the filenames of the mapping files
# supports multiple maps-files
sub get_maps_files {
  $_[0] =~ /:(\/[^,\s]*)(.*)/;
  (my $returnvalue, my $recurse) = ( $1, $2 );

  return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         &get_maps_files($recurse)
	     :
	         ()
           )
}

 
### get_maps($maps_name) : \@maps
sub get_maps {
  if (!@maps_cache) {
    my @maps_files = &get_maps_files(&get_current_value($_[0]));
    my $number = 0;

    foreach $maps_file (@maps_files) {
      my $i = 0;
      open(MAPS, $maps_file);
      while (<MAPS>) {
        s/^#.*$//g;	# remove comments
        s/\r|\n//g;	# remove newlines

        if (/^\s*([^\s]+)\s+([^#]*)/) {
          $number++;
          my %map;
          $map{'name'} = $1;
          $map{'value'} = $2;
          $map{'line'} = $i;
          $map{'map_file'} = $maps_file;
          $map{'number'} = $number;
          push(@maps_cache, \%map);
        }

        $i++;
      }
      close(MAPS);
    }
  }

  return \@maps_cache;
}


sub generate_map_edit {
  if (&get_current_value($_[0]) eq "") {
    print ("<P>No map is currently defined. Define a map first, then you can edit it.<br>");
    &ryanFooter("canonical.cgi");
    exit;
  }

  my $mappings = &get_maps($_[0]);

  if ($#{$mappings} ne -1) {
	
    print "<CENTER><table width=620><TR><TD COLSPAN=2>$_[1]</TD></TR>\n";
    #print "<CENTER><table width=620>\n";
    print "<tr><td width=50% valign=top>\n";
    print "<table border=0 CELLPADDING=0 CELLSPACING=0 width=100%>\n";
    print "<tr $tb> <td><DIV CLASS=thead><b>$text{'mapping_name'}</b></td> ",
          "<td><DIV CLASS=thead><b>$text{'mapping_value'}</b></td> </tr>\n";

    my $split_index = int(($#{$mappings})/2);
    my $i = -1;
	
    foreach $map (@{$mappings}) {
      print "<tr $cb>\n";
      print "<td><DIV CLASS=nbody><a href=\"edit_mapping.cgi?num=$map->{'number'}&map_name=$_[0]\">$map->{'name'}</a></td>\n";
      print "<td><DIV CLASS=nbody>$map->{'value'}</td>\n</tr>\n";
      $i++;

      if ($i == $split_index) {
        print "</table></td><td width=50% valign=top>\n";
        print "<table border=0 CELLPADDING=0 CELLSPACING=0 width=100%>\n";
        print "<tr $tb> <td><DIV CLASS=thead><b>$text{'mapping_name'}</b></td> ",
              "<td><DIV CLASS=thead><b>$text{'mapping_value'}</b></td> </tr>\n";
      }
    }
	
    print "</tr></td></table>\n";
    print "</table></CENTER>\n";
  }


# new form

    print "<form action=edit_mapping.cgi>\n";
    print "<CENTER><table border=0 cellpadding=10 CELLSPACING=0  width=520>\n";
    print "<tr $cb>\n";
    print "  <td ALIGN=RIGHT>\n";
    print "    <input type=hidden name=\"map_name\" value=\"$_[0]\">";
    print "    <input type=submit value=\"$text{'new_mapping'}\">\n";
    print "  </td>\n";
    print "  <td width=\"99%\">$text{'new_mappingmsg'}</td>\n";
    print "</tr>\n";
    print "</table></CENTER></form>\n";

}


# init_new_mapping($maps_parameter) : $number
# gives a new number of mapping
sub init_new_mapping
{
    $maps = &get_maps($_[0]);

    my $max_number = 0;

    foreach $trans (@{$maps})
    {
	if ($trans->{'number'} > $max_number) { $max_number = $trans->{'number'}; }
    }
    
    return $max_number+1;
}

################################################################################
### Guardian Digital Subroutines ###############################################
################################################################################

################################################################################
### &get_domains() #############################################################
################################################################################
sub get_domains() {

  open VIRTUAL, "/etc/postfix/virtual";
    while (<VIRTUAL>) {
      chomp $_;
      if ($_ =~ m/^(.*)\s+VIRTUALDOMAIN$/) { push @domains, $1; }
    }
  close VIRTUAL;

  return @domains;
}



################################################################################
### &save_mapping() ############################################################
################################################################################
sub save_mapping() {
  local ($fcn, $domain, $index, $source, $dest);
  ($fcn, $domain, $index, $source, $dest) = @_;

  $i = 0;
  open VIRTUAL, "/etc/postfix/virtual";
    while (<VIRTUAL>) {
      chomp $_;
      if ($_ =~ m/^$domain\s+VIRTUALDOMAIN$/) { $start_line = $i; }
      $virtusertable[$i++] = $_;
    }
  close VIRTUAL;


  ### Get lock and open virtual file.
  &obtain_lock("/etc/postfix/virtual");
  open VIRTUAL, ">/etc/postfix/virtual";

  ### Perform our function.
  ### We want to delete a mapping.
  if ($fcn == 0) {

    ### Generate our offset.
    $offset = $start_line + $index + 1;

    ### Splice in our new_value.
    splice @virtusertable, $offset, 1;

    ### Print file back.
    for ($j = 0; $j < @virtusertable; $j++) {
      print VIRTUAL "$virtusertable[$j]\n"; 
    }

  }

  ### We want to update a mapping.
  if ($fcn == 1) {

    ### Generate our offset.
    $offset = $start_line + $index + 1;

    ### Splice in our new_value.
    splice @virtusertable, $offset, 1, "$source\t$dest";

    ### Print file back.
    for ($j = 0; $j < @virtusertable; $j++) {
      print VIRTUAL "$virtusertable[$j]\n"; 
    }

  }

  ### We want to insert a new mapping.
  elsif ($fcn == 2) {

    ### Insert our new entry right below the virtual.domain definition.
    foreach $v (@virtusertable) {
      if ($v =~ m/^$domain\s+VIRTUALDOMAIN$/) {
        print VIRTUAL "$v\n";
        print VIRTUAL "$source\t$dest\n";
      }
      else { print VIRTUAL "$v\n"; }
    }

  }


  ### Close the door and wash our hands.
  close VIRTUAL;
  &release_lock("/etc/postfix/virtual");
}


################################################################################
### &add_domain() ##############################################################
################################################################################
sub add_domain() {
  local ($domain, $defrec);
  ($domain, $defrec) = @_;

  ### Add the entry to /etc/postfix/virtual.
  $lref = &read_file_lines("/etc/postfix/virtual");
    push @$lref, "\n$domain\tVIRTUALDOMAIN";
    push @$lref, "\@$domain\t$defrec\n";
  &flush_file_lines("/etc/postfix/virtual");

  ### Add the domain to $relay_domains.
  &add_relaydomain($domain);
}


################################################################################
### &delete_domain() ###########################################################
################################################################################
sub delete_domain() {
  my ($domain) = @_;
  my $in_dom = 0;


  ### Delete the entry from /etc/postfix/virtual.
  $lref = &read_file_lines("/etc/postfix/virtual");
    for ($i = 0; $i < @$lref; $i++) {
      if (@$lref[$i] =~ m/^$domain\s+VIRTUALDOMAIN$/) {
        $in_dom = 1; $start = $i;
      }
      elsif (($in_dom == 1) && (@$lref[$i] eq "")) { $in_dom = 0; $end = $i; }
    }

    splice @$lref, $start, $end - $start;

  &flush_file_lines("/etc/postfix/virtual");

  ### Delete the domain from $relay_domains.
  &delete_relaydomain($domain);
}


################################################################################
### &get_mappings() ############################################################
################################################################################
sub get_mappings() {
  local ($domain);
  ($domain) = @_;

  $in_domain = 0;
  $i = 0;
  open VIRTUAL, "/etc/postfix/virtual";
    while (<VIRTUAL>) {
      chomp $_;
      if ($_ =~ m/^$domain\s+VIRTUALDOMAIN$/) { $in_domain = 1; }
      elsif ($_ eq "") { $in_domain = 0; }
      elsif ($in_domain) {
        ($src, $dest) = split /\s+/, $_;
        $mappings[$i]->{'index'}  = $i;
        $mappings[$i]->{'source'} = $src;
        $mappings[$i++]->{'dest'} = $dest;
      }
    }
  close VIRTUAL;

  return @mappings;
}

################################################################################
### &get_transport_map() #######################################################
################################################################################
sub get_transport_map() {
  local (@transport, $i);
  $i = 0;

  open TRANSPORT, "/etc/postfix/transport";
    while (<TRANSPORT>) {
      chomp $_;

      if ($_ =~ m/^#/) { next; }
      elsif ($_ =~ m/^([^\ ]+)\ ([^\ ]+)$/) {
        $transport[$i]->{'domain'}      = $1; 
        $transport[$i++]->{'destination'} = $2; 
      }
    }
  close TRANSPORT;

  return @transport;

}






################################################################################
### modify_relaydomain #########################################################
### Modify a domain in $relay_domains. #########################################
################################################################################
sub modify_relaydomain() {
  my ($del_domain, $add_domain) = @_;
  my @domains = &get_relaydomains();

  foreach (@domains) {
    if ($_ ne $del_domain) {
      push @update_domains, $_;
    }
    else { push @update_domains, $add_domain; }
  }

  ### Save the updated domain list.
  &save_relaydomains(@update_domains);
}

################################################################################
### add_relaydomain ############################################################
### Add a domain to $relay_domains. ############################################
################################################################################
sub add_relaydomain() {
  my ($add_domain) = @_;
  my @domains = &get_relaydomains();

  ### Add the target domain to the list of domains.
  push @domains, $add_domain;
 
  ### Save the updated domain list.
  &save_relaydomains(@domains);
}

################################################################################
### delete_relaydomain #########################################################
### Delete a domain from $relay_domains. #######################################
################################################################################
sub delete_relaydomain() {
  my ($del_domain) = @_;
  my @domains = &get_relaydomains();
  
  ### Get the domains and delete our target domain.
  foreach (@domains) { if ($_ ne $del_domain) { push @update_domains, $_; } }


  ### Save the updated domain list.
  &save_relaydomains(@update_domains);
}

################################################################################
### get_relaydomains ###########################################################
### Read the relay domains from disk. ##########################################
################################################################################
sub get_relaydomains() {
  
  ### Read the file in.
  $lref = &read_file_lines("/etc/postfix/main.cf");

  foreach $line (@$lref) {
    if ($line =~ m/^relay_domains\ =\ (.*)$/) {

      ### We have the relay domain line, so split it into domains.
      @relay_domains = split /[\ \,]+/, $1;

      ### Iterate through and filter out the $mydestination variable.
      foreach (@relay_domains) {
        if ($_ ne "\$mydestination") { push @rv, $_; }
      }
    }
  }

  ### Write the file out.
  &flush_file_lines();

  ### Return our domains.
  return @rv;
}


################################################################################
### save_relaydomains ##########################################################
### Write the relay domains back to disk. ######################################
################################################################################
sub save_relaydomains() {
  my (@domains) = @_;

  ### Build our new relay_domains string.
  $relay_domains = "relay_domains = \$mydestination";
  foreach (@domains) { $relay_domains .= ", $_"; }

  ### Read the file in.
  $lref = &read_file_lines("/etc/postfix/main.cf");

  foreach $line (@$lref) {
    if ($line =~ m/^relay_domains\ =\ (.*)$/) { $line = $relay_domains; }
  }

  ### Write the file out.
  &flush_file_lines();
}




return 1;
