#!/usr/bin/perl

  do "/usr/webtool/web-lib.pl";


  ### Check command line arguments
  usage() if (@ARGV != 3);

  ($config, $user, $pass) = @ARGV;


  if (!-d $config) {
    print STDERR "The config directory $config does not exist\n";
    exit 2;
  }


  ### ########################### ###
  ### Update the miniserv config. ###
  ### ########################### ###
  if (!open(CONF, "$config/miniserv.conf")) {
    print STDERR "Failed to open $config/miniserv.conf : $!\n";
    print STDERR "Maybe $config is not the Webmin config directory.\n";
    exit 3;
  }


  while(<CONF>) { if (/^([^=]+)=(\S+)/) { $config{$1} = $2; } }

  close(CONF);


  ### ######################## ###
  ### Update the users config. ###
  ### ######################## ###
  if (!open(USERS, $config{'userfile'})) {
    print STDERR "Failed to open Webmin users file $config{'userfile'} : $!\n";
    exit 4;
  }

  while(<USERS>) { if (/^([^:]+):(\S+)/) { $users{$1} = $2; } }

  close(USERS);



  if (!defined($users{$user})) {
    print STDERR "The Webmin user $user does not exist\n";
    exit 5;
  }

  ### ########################################################### ###
  ### Make our encrypted password and write it to the users file. ###
  ### ########################################################### ###
  $salt = substr(time(), 0, 2);
  $users{$user} = crypt($pass, $salt);

  if (!open(USERS, "> $config{'userfile'}")) {
    print STDERR "Failed to open Webmin users file $config{'userfile'} : $!\n";
    exit 6;
  }

  foreach $k (keys %users) { print USERS "$k:$users{$k}\n"; }
  close(USERS);


  ### ######################################################### ###
  ### Now the fun part.  Update the root mysql user's password. ###
  ### ######################################################### ###
  open FILE, "/etc/webtool/SystemManager/config";
    while (<FILE>) { if ($_ =~ m/^pass\=(.*)$/) { $mysql_pass = $1; } }
  close FILE;

    $lref = &read_file_lines("/etc/webtool/SystemManager/config");
    foreach (@$lref) { if ($_ =~ m/^pass\=(.*)$/) { $_ = "pass=$pass"; } }
    &flush_file_lines("/etc/webtool/SystemManager/config");
   

    use DBI;

    ### Setup DBMS variables;
    $database = "mysql"; $host = "localhost"; $user = "root";

    ### Connect to the database;
    my $dbh = DBI->connect("DBI:mysql:database=$database;host=$host", "$user",
                           "$mysql_pass");

    ### No go, let's try without a password for grins.
    if (!$dbh) {
      $dbh = DBI->connect("DBI:mysql:database=$database;host=$host", "$user");
    }

    ### We connected properly...
    if ($dbh) {
      ### Prepare our query.
      $sth = $dbh->prepare("UPDATE user
                            SET Password = PASSWORD('$pass')
                            WHERE User = 'root'");
      ### Execute it!
      $sth->execute();
      $sth->finish();

      ### Reload grant tables.
      $sth = $dbh->prepare("FLUSH PRIVILEGES");
      $sth->execute();
      $sth->finish();

      ### Disconnect from the DBMS.
      $dbh->disconnect();
    }


  ### ####################################################### ###
  ### You thought THAT was fun?  Now let's make LIDS happy... ###
  ### ####################################################### ###
  if (! system "rpm -q lids-base > /dev/null 2>&1") {
    use Crypt::RIPEMD160;

    ### Setup our two objects.
    my $first_hash = new Crypt::RIPEMD160;
    my $second_hash = new Crypt::RIPEMD160;

    ### Add the $password to the initial hash, then get the digest string.
    $first_hash->add($pass);
    $digest     = $first_hash->digest();;
    $digest_str = unpack "H*", $digest;

    ### Add the hashed $password to the second hash, then get the digest string.
    $second_hash->add($digest_str);
    $final_digest  = $second_hash->digest();
    $lids_pw       = unpack "H*", $final_digest;


  ### Okay, now we have our password.  Next we must disable LIDS.
  my $old_hash = new Crypt::RIPEMD160;
  $old_hash->add($mysql_pass);
  $digest      = $old_hash->digest();;
  $OLD_PASS    = unpack "H*", $digest;


    ### Turn off LIDS.
    $out = `/sbin/lidsadm_gd -Q -- -LIDS $OLD_PASS 2>&1`;

      ### Write the new password.
      open LIDS_PW, ">/etc/lids/lids.pw";
        print LIDS_PW $lids_pw;
      close LIDS_PW;

      ### Generate the ripemd-160 hash of the new password.
      $encpass = `echo -n "$pass" | openssl dgst -rmd160`;

    ### Now reload, then re-enable LIDS.
    $out = `/sbin/lidsadm_gd -Q -- +RELOAD_CONF $OLD_PASS 2>&1`;
    $out = `/sbin/lidsadm_gd -Q -- +LIDS $encpass 2>&1`;
  }

  ### Print a happy success message.
  # print "Updated password of Webmin user $user ($mysql_pass)\n";
  # print "Updated password of Webmin user $user\n";



################################################################################
sub usage {
  print STDERR <<EOF;
Usage: /usr/webtool/changepass.pl <config-dir> <login> '<password>'

Example Usage:
  /usr/webtool/changepass.pl /etc/webtool admin 'lock&%box'

EOF

  exit 1;
}

