# web-lib.pl
# Common functions and definitions for web admin programs

# Vital libraries
use Socket;


# Configuration and spool directories
if (!defined($ENV{'WEBMIN_CONFIG'})) { die "WEBMIN_CONFIG not set"; }
$config_directory = $ENV{'WEBMIN_CONFIG'};
if (!defined($ENV{'WEBMIN_VAR'})) { die "WEBMIN_VAR not set"; }
$var_directory = $ENV{'WEBMIN_VAR'};

# read_file(file, &assoc, [&order], [lowercase])
# Fill an associative array with name=value pairs from a file
sub read_file
{


open(ARFILE, $_[0]) || return 0;
while(<ARFILE>) {
	s/\r|\n//g;
        if (!/^#/ && /^([^=]+)=(.*)$/) {
		$_[1]->{$_[3] ? lc($1) : $1} = $2;
		push(@{$_[2]}, $1) if ($_[2]);
        	}
        }
close(ARFILE);
return 1;
}
 
# write_file(file, array)
# Write out the contents of an associative array as name=value lines
sub write_file
{
local($arr);
$arr = $_[1];
open(ARFILE, "> $_[0]");
foreach $k (keys %$arr) {
        print ARFILE "$k=$$arr{$k}\n";
        }
close(ARFILE);
}

# html_escape
# Convert &, < and > codes in text to HTML entities
sub html_escape
{
local($tmp);
$tmp = $_[0];
$tmp =~ s/&/&amp;/g;
$tmp =~ s/</&lt;/g;
$tmp =~ s/>/&gt;/g;
$tmp =~ s/\"/&#34;/g;
return $tmp;
}

# obtain_lock
# Get a lock on a file, or wait.
# If the file is locked, and the locking process no longer exists ignore the
# lock and carry on.
sub obtain_lock {
  while(stat("$_[0].lock")) {
	sleep(1);
	open(LOCK, "$_[0].lock");
	chop($lpid = <LOCK>);
	close(LOCK);
	if ($lpid && !kill(0, "/proc/$lpid")) {
		# The process holding this lock is gone!
		# print STDERR "Process $lpid does not exist\n";
		last;
		}
	}
  open(LOCK,">$_[0].lock");
  print LOCK "$$\n";
  close(LOCK);
}

# release_lock
# Release the lock we are holding on a file
sub release_lock {
  unlink("$_[0].lock");
}

# test_lock
# Is this file locked?
sub test_lock
{
if (-r "$_[0].lock") {
	open(LOCK, "$_[0].lock");
	<LOCK> =~ /(\d+)/;
	close(LOCK);
	if ($1 && kill(0, $1)) { return 1; }
	}
return 0;
}

# tempname
# Returns a mostly random te0porary file name
sub tempname {
  while(1) {
    local @st = lstat("/tmp/.webtool");
    last if (!$st[4] && !$st[5] && $st[2] & 0x4000 &&
            ($st[2] & 0777) == 0700);

    if (@st) {
      unlink("/tmp/.webtool") || system("rm -rf /tmp/.webtool");
    }

    mkdir("/tmp/.webtool", 0700) || next;
    chown(0, 0, "/tmp/.webtool");
  }

  use POSIX qw(tmpnam);
  umask 0077;
  my ($temp) = (POSIX::tmpnam() =~ m/^\/tmp\/([A-Za-z0-9]+)$/);

  return "/tmp/.webtool/$temp";
}

# trunc
# Truncation a string to the shortest whole word less than the given width
sub trunc {
  local($str,$c);
  if (length($_[0]) <= $_[1])
    { return $_[0]; }
  $str = substr($_[0],0,$_[1]);
  do {
    $c = chop($str);
    } while($c !~ /\S/);
  return $str;
}

# indexof
# Returns the index of some value in an array, or -1
sub indexof {
  local($i);
  for($i=1; $i <= $#_; $i++) {
    if ($_[$i] eq $_[0]) { return $i - 1; }
    }
  return -1;
  }

# unique
# Returns the unique elements of some array
sub unique
{
local(%found, @rv, $e);
foreach $e (@_) {
	if (!$found{$e}++) { push(@rv, $e); }
	}
return @rv;
}

# sysprint(handle, [string]+)
sub sysprint
{
local($str, $fh);
$str = join('', @_[1..$#_]);
$fh = $_[0];
syswrite $fh, $str, length($str);
}

# check_ipaddress(ip)
# Check if some IP address is properly formatted
sub check_ipaddress
{
return $_[0] =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ &&
	$1 >= 0 && $1 <= 255 &&
	$2 >= 0 && $2 <= 255 &&
	$3 >= 0 && $3 <= 255 &&
	$4 >= 0 && $4 <= 255;
}

# generate_icon(image, title, link)
sub generate_icon
{
if ($_[2]) {
	print "<table><tr><td>\n",
	      "<a href=\"$_[2]\"><img src=\"$_[0]\" border=0 ",
	      "width=48 height=48></a></td></tr></table>\n";
	print "<a href=\"$_[2]\">$_[1]</a>\n";
	}
else {
	print "<table><tr><td>\n",
	      "<img src=\"$_[0]\" border=0 width=48 height=48>",
	      "</td></tr></table>\n$_[1]\n";
	}
}

# urlize
# Convert a string to a form ok for putting in a URL
sub urlize {
  local($tmp, $tmp2, $c);
  $tmp = $_[0];
  $tmp2 = "";
  while(($c = chop($tmp)) ne "") {
	if ($c !~ /[A-z0-9]/) {
		$c = sprintf("%%%2.2X", ord($c));
		}
	$tmp2 = $c . $tmp2;
	}
  return $tmp2;
}

# include
# Read and output the named file
sub include
{
open(INCLUDE, $_[0]);
while(<INCLUDE>) {
	print;
	}
close(INCLUDE);
}

# copydata
# Read from one file handle and write to another
sub copydata
{
local($line, $out, $in);
$out = $_[1];
$in = $_[0];
while($line = <$in>) {
	print $out $line;
	}
}

# ReadParseMime
# Read data submitted via a POST request using the multipart/form-data coding
sub ReadParseMime
{
local($boundary,$line,$foo);
$ENV{CONTENT_TYPE} =~ /boundary=(.*)$/;
$boundary = $1;
<STDIN>;	# skip first boundary
while(1) {
	$name = "";
	# Read section headers
	while(1) {
		$line = <STDIN>;
		chop($line); chop($line);	# lose /r/n
		if (!$line) { last; }
		elsif ($line =~ /^Content-Disposition: form-data(.*)/) {
			$rest = $1;
			while ($rest =~ /([a-zA-Z]*)=\"([^\"]*)\"(.*)/) {
				if ($1 eq name) {
					$name = $2;
					}
				else {
					$foo = $name . "_$1";
					$in{$foo} = $2;
					}
				$rest = $3;
				}
			}
		elsif ($line =~ /^Content-Type: (.*)/) {
			$foo = $name . "_content_type";
			$in{$foo} = $1;
			}
		}
	# Read data
	$in{$name} .= "\0" if ($in{$name});
	while(1) {
		$line = <STDIN>;
		if (!$line) { return; }
		if (index($line,"$boundary") != -1) { last; }
		$in{$name} .= $line;
		}
	chop($in{$name}); chop($in{$name});
	if (index($line,"$boundary--") != -1) { last; }
	}
}

# ReadParse([&assoc])
# Fills the given associative array with CGI parameters, or uses the global
# %in if none is given. Also sets the global variables $in and @in
sub ReadParse
{
local $a = $_[0] ? $_[0] : \%in;
local $i;
if ($ENV{'REQUEST_METHOD'} eq 'POST') {
	read(STDIN, $in, $ENV{'CONTENT_LENGTH'});
	}
else {
	$in = $ENV{'QUERY_STRING'};
	}
@in = split(/\&/, $in);
foreach $i (@in) {
	local ($k, $v) = split(/=/, $i, 2);
	$k =~ s/\+/ /g; $k =~ s/%(..)/pack("c",hex($1))/ge;
	$v =~ s/\+/ /g; $v =~ s/%(..)/pack("c",hex($1))/ge;
	$a->{$k} = defined($a->{$k}) ? $a->{$k}."\0".$v : $v;
	}
}

# PrintHeader
# Outputs the HTTP header for HTML
sub PrintHeader
{
# print "pragma: no-cache\n" if ($pragma_no_cache);
# print "pragma: no-cache\n";
print "Content-type: text/html",
	$_[0] ? "; Charset=$_[0]" : "","\n\n";
}

# header(title, image, [help], [config], [nomodule], [nowebmin], [rightside],
#	 [header], [body])
# Output a page header with some title and image. The header may also
# include a link to help, and a link to the config page.
# The header will also have a link to to webmin index, and a link to the
# module menu if there is no config link

sub header {
  local($l, $ll, %access, $lang);

  foreach $l (&list_languages()) {
    $lang = $l if ($l->{'lang'} eq $current_lang);
  }

  &PrintHeader($lang->{'charset'});

  print "<html>\n";

  if (@_ > 0) {
	print "<head>\n";
	print "<title>$_[0]</title>\n";
	print $_[7] if ($_[7]);
	print "<SCRIPT LANGUAGE=\"JavaScript\">\n";
	printf "defaultStatus=\"%s\@%s logged into the WebTool v%s \";\n",
		$ENV{'REMOTE_USER'},
                &get_system_hostname(),
		&get_webmin_version();
	print "</SCRIPT>\n\n";

        print "<STYLE TYPE=\"text/css\">\n";

        print "  BODY { background-image: url(/images/backgrn.gif); }\n";
	print "  A:link { text-decoration: none; }\n";
	print "  A:visited { text-decoration: none; }\n";
	print "  A:active { text-decoration: none }\n";
        print "  A:hover { color: #D3ACDA; }\n";


        ### print "  DIV,BODY,TD,TH,P,FONT,INPUT,LI,UL,FORM,BLOCKQUOTE,A,TR,";
        ### print "TABLE,I { font-family: Arial, Helvetica, Sans-Serif; ";

        ### print "  BODY,TH,P,FONT,LI,UL,BLOCKQUOTE,A,";
        ### print "I { font-family: Arial, Helvetica, Sans-Serif; ";

        print "  BODY,TH,P,FONT,LI,UL,BLOCKQUOTE,A,";
        print "TABLE,I { font-family: Arial, Helvetica, Sans-Serif; ";
          print "font-size: 10pt }\n";

        print "  PRE { font-family: Courier }\n";
        print "  .thead { font-family: Arial, Helvetica, Sans-Serif; ";
          print "font-size: 10pt; color: #ffffff; margin-left: 10px; }\n";

        print "  .ttitle { font-family: Arial, Helvetica, Sans-Serif; ";
          print "font-size: 14pt; color: #ffffff; margin-left: 10px; }\n";

        print "  .rhead { font-family: Arial, Helvetica, Sans-Serif; ";
          print "font-size: 10pt; color: #ffffff; margin-right: 10px; }\n";

        print "  .nbody { font-family: Arial, Helvetica, Sans-Serif; ";
          print "font-size: 10pt; margin-left: 10px; margin-right: 10px; }\n";

        print "  .tbody { margin-left: 10px; }\n";
        print "  .toolfont { font-family: Arial, Helvetica, Sans-Serif; ";
        print "  font-size: 10pt; }\n";

        print "</STYLE>\n";

	print "</head>\n";
	}


local $bgcolor = defined($gconfig{'cs_page'}) ? $gconfig{'cs_page'} : "ffffff";
local $link = defined($gconfig{'cs_link'}) ? $gconfig{'cs_link'} : "0000ee";
local $text = defined($gconfig{'cs_text'}) ? $gconfig{'cs_text'} : "000000";

print "<body bgcolor=#FFFFFF link=#$link vlink=#$link text=#$text $_[8]>\n";

  if (@_ > 1) {
print "<TABLE WIDTH=\"707\" BORDER=\"0\">\n";
print "<TR>\n";
print "<TD WIDTH=\"25\">&nbsp;</td>\n";
print "<TD WIDTH=\"647\">\n";

    print "\n<table border=0 width=100%>\n  <tr>\n";
    print "  </td>\n\n";


    local $title = $_[0];
    $title =~ s/&auml;//g;
    $title =~ s/&ouml;//g;
    $title =~ s/&uuml;//g;


    ### [rwm] Print out our image instead.
    if ($ENV{'REQUEST_URI'} =~ m/Security/) {
      $img = "/images/header-Security.gif"; $hdr_url = "/Security/";
    }
    elsif ($ENV{'REQUEST_URI'} =~ m/StatusMonitor/) {
      $img = "/images/header-StatusMonitor.gif"; $hdr_url = "/StatusMonitor/";
    }
    elsif ($ENV{'REQUEST_URI'} =~ m/SystemBackup/) {
      $img = "/images/header-SystemBackup.gif"; $hdr_url = "/SystemBackup/";
    }
    elsif ($ENV{'REQUEST_URI'} =~ m/SystemManager/) {
      $img = "/images/header-SystemManager.gif"; $hdr_url = "/SystemManager/";
    }
    elsif ($ENV{'REQUEST_URI'} =~ m/VHost/) {
      $img = "/images/header-VHost.gif"; $hdr_url = "/VHost/";
    }
    elsif ($ENV{'REQUEST_URI'} =~ m/gdupdate/) {
      $img = "/images/header-gdupdate.gif"; $hdr_url = "/gdupdate/";
    }
    else { $img = "/images/head.gif"; $hdr_url = "/"; }

    print "  <td align=center width=70%>";
    print "        <IMG SRC=\"$img\" USEMAP=\"#header\"BORDER=0>\n";
    print "  </td>\n\n";
    print "  </td>\n</tr>\n</table>\n\n";
  }

}

# footer(page, name)
# Output a footer for returning to some page
sub footer
{
if (@_) {
	local $url = $_[0];
	$url = "/?cat=$module_info{'category'}" if ($url eq '/');
	print "<a href=\"$url\"><img alt=\"<-\" align=middle border=0 src=/images/left.gif></a>\n";
	print "&nbsp;&nbsp;<a href=\"$url\">",&text('main_return', $_[1]),
	      "</a><p>\n";
	}
print "</body></html>\n";
}

# redirect
# Output headers to redirect the browser to some page
sub redirect
{
local($port, $prot, $url);
$port = $ENV{'SERVER_PORT'} == 443 && uc($ENV{'HTTPS'}) eq "ON" ? "" :
	$ENV{'SERVER_PORT'} == 80 && uc($ENV{'HTTPS'}) ne "ON" ? "" :
		":$ENV{'SERVER_PORT'}";
$prot = uc($ENV{'HTTPS'}) eq "ON" ? "https" : "http";
if ($_[0] =~ /^(http|https|ftp|gopher):/) {
	$url = $_[0];
	}
elsif ($_[0] =~ /^\//) {
	$url = "$prot://$ENV{'SERVER_NAME'}$port$_[0]";
	}
else {
	$ENV{'SCRIPT_NAME'} =~ /^(.*)\/[^\/]+$/;
	$url = "$prot://$ENV{'SERVER_NAME'}$port$1/$_[0]";
	}
print "Location: $url\n\n";
}

# kill_byname(name, signal)
# Use the command defined in the global config to find and send a signal
# to a process matching some name
sub kill_byname
{
local(@pids);
@pids = &find_byname($_[0]);
if (@pids) { kill($_[1], @pids); }
}

# find_byname(name)
# Finds a process by name, and returns a list of matching PIDs
sub find_byname
{
local($cmd, @pids);
$cmd = $gconfig{'find_pid_command'};
$cmd =~ s/NAME/$_[0]/g;
@pids = split(/\n/, `$cmd`);
return @pids;
}

# error([message]+)
# Display an error message and exit. The variable $whatfailed must be set
# to the name of the operation that failed.
sub error
{
&header($text{'error'}, "");
print "<CENTER><TABLE BORDER=\"0\" WIDTH=\"620\">\n";
print "<TR><TD>\n";
print "<h3>",($whatfailed ? "$whatfailed : " : ""),@_,"</h3>\n";
print "</TD></TR>\n";
print "</TABLE>\n\n";

&ryanFooter();
exit;
}

# error_setup(message)
# Register a message to be prepended to all error strings
sub error_setup
{
$whatfailed = $_[0];
}

# wait_for(handle, regexp, regexp, ...)
# Read from the input stream until one of the regexps matches..
sub wait_for
{
local($hit, $c, $i, $sw, $rv, $ha); undef($wait_for_input);
#print STDERR "wait_for(",join(",", @_),")\n";
$ha = $_[0];
$code =
"undef(\$hit);\n".
"while(1) {\n".
" if ((\$c = getc($ha)) eq \"\") { return -1; }\n".
" \$wait_for_input .= \$c;\n";
#" \$wait_for_input .= \$c;\nprint STDERR \$wait_for_input,\"\\n\";";
for($i=1; $i<@_; $i++) {
        $sw = $i>1 ? "elsif" : "if";
        $code .= " $sw (\$wait_for_input =~ /$_[$i]/i) { \$hit = $i-1; }\n";
        }
$code .=
" if (defined(\$hit)) {\n".
"  \@matches = (-1, \$1, \$2, \$3, \$4, \$5, \$6, \$7, \$8, \$9);\n".
"  return \$hit;\n".
"  }\n".
" }\n";
$rv = eval $code;
if ($@) { &error("wait_for error : $@\n"); }
return $rv;
}

# fast_wait_for(handle, string, string, ...)
sub fast_wait_for
{
local($inp, $maxlen, $ha, $i, $c, $inpl);
for($i=1; $i<@_; $i++) {
	$maxlen = length($_[$i]) > $maxlen ? length($_[$i]) : $maxlen;
	}
$ha = $_[0];
while(1) {
	if (($c = getc($ha)) eq "") {
		&error("fast_wait_for read error : $!");
		}
	$inp .= $c;
	if (length($inp) > $maxlen) {
		$inp = substr($inp, length($inp)-$maxlen);
		}
	$inpl = length($inp);
	for($i=1; $i<@_; $i++) {
		if ($_[$i] eq substr($inp, $inpl-length($_[$i]))) {
			return $i-1;
			}
		}
	}
}

# has_command(command)
# Returns the full path if some command is in the path, undef if not
sub has_command
{
local($d);
if (!$_[0]) { return undef; }
if ($_[0] =~ /^\//) { return (-x $_[0]) ? $_[0] : undef; }
foreach $d (split(/:/ , $ENV{PATH})) {
	if (-x "$d/$_[0]") { return "$d/$_[0]"; }
	}
return undef;
}

@make_date_marr = ("Jan", "Feb", "Mar", "Apr", "May", "Jun",
	 	   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");

# make_date(seconds)
# Converts a Unix date/time in seconds to a human-readable form
sub make_date
{
local(@tm);
@tm = localtime($_[0]);
return sprintf "%d/%s/%d %2.2d:%2.2d",
		$tm[3], $make_date_marr[$tm[4]], $tm[5]+1900, $tm[2], $tm[1];
}

# file_chooser_button(input, type, [form])
# Return HTML for a file chooser button, if the browser supports Javascript.
# Type values are 0 for file or directory, or 1 for directory only
sub file_chooser_button {
  local($form);
  $form = @_ > 2 ? $_[2] : 0;

  return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"/chooser.cgi?type=$_[1]&file=\"+ifield.value, \"chooser\", \"toolbar=no,menubar=no,scrollbar=no,width=400,height=300\"); chooser.ifield = ifield' value=\"...\">\n";
}

# read_acl(&array, &array)
# Reads the acl file into the given associative arrays
sub read_acl
{
local($user, $_, @mods);
open(ACL, &acl_filename());
while(<ACL>) {
	if (/^(\S+):\s*(.*)/) {
		local(@mods);
		$user = $1;
		@mods = split(/\s+/, $2);
		if ($_[0]) { foreach $m (@mods) { ${$_[0]}{$user,$m}++; } }
		if ($_[1]) { $_[1]->{$user} = \@mods; }
		}
	}
close(ACL);
}

# acl_filename()
# Returns the file containing the webmin ACL
sub acl_filename
{
return "$config_directory/webmin.acl";
}

# acl_check()
# Does nothing, but kept around for compatability
sub acl_check
{
}

# get_miniserv_config(&array)
# Store miniserv configuration into the given array
sub get_miniserv_config
{
return &read_file($ENV{'MINISERV_CONFIG'}, $_[0]);
}

# put_miniserv_config(&array)
# Store miniserv configuration from the given array
sub put_miniserv_config
{
&write_file($ENV{'MINISERV_CONFIG'}, $_[0]);
}



### restart_miniserv()
### Send a HUP signal to miniserv
#  local($pid, %miniserv, $addr, $i);
#  &get_miniserv_config(\%miniserv) || return;
#  $miniserv{'inetd'} && return;
#
#  open(PID, $miniserv{'pidfile'}) || &error("Failed to open pid file");
#    chop($pid = <PID>);
#  close(PID);
#
#  if (!$pid) { &error("Invalid pid file"); }
#  kill('HUP', $pid);
#
#  ### Wait for miniserv to come back up.
#  $addr = inet_aton($miniserv{'bind'} ? $miniserv{'bind'} : "127.0.0.1");
#
#  for($i=0; $i<20; $i++) {
#    sleep(1);
#
#    socket(STEST, PF_INET, SOCK_STREAM, getprotobyname("tcp"));
#    $rv = connect(STEST, sockaddr_in($miniserv{'port'}, $addr));
#    close(STEST);
#
#    if ($rv) { last; }
#  }
#
#  if ($i == 20) { &error("Failed to restart Webmin server!"); }
#
sub restart_miniserv {

  ### Clean and save the environment.
  my %ENV_RESTORE;
  %ENV_RESTORE  =  &clean_env();

    ### Restart the daemon.
    system "/etc/init.d/webtool restart > /dev/null 2>&1";

  ### Restore the environment.
  &restore_env(%ENV_RESTORE);
}



# check_os_support(&minfo)
sub check_os_support
{
local @oss = split(/\s+/, $_[0]->{'os_support'});
if (!@oss)
	{ return 1; }
elsif (&indexof($gconfig{'os_type'}, @oss) != -1)
	{ return 1; }
elsif (&indexof("$gconfig{'os_type'}/$gconfig{'os_version'}", @oss) != -1)
	{ return 1; }
else
	{ return 0; }
}

# http_download(host, port, page, destfile)
# Download data from a HTTP url to a local file
sub http_download
{
$SIG{ALRM} = "download_timeout";
alarm(60);
if ($gconfig{'http_proxy'} =~ /^http:\/\/(\S+):(\d+)/ && !&no_proxy($_[0])) {
	# going through proxy
	&open_socket($1, $2, "SOCK");
	print SOCK "GET http://$_[0]:$_[1]$_[2] HTTP/1.0\r\n";
	}
else {
	# can connect directly
	&open_socket($_[0], $_[1], "SOCK");
	print SOCK "GET $_[2] HTTP/1.0\r\n";
	}
alarm(0);
print SOCK "Host: $_[0]\r\n";
print SOCK "User-agent: Webmin\r\n";
print SOCK "\r\n";
&complete_http_download("SOCK", $_[3]);
}

# complete_http_download(socket, destfile)
# Do a HTTP download, after the headers have been sent
sub complete_http_download
{
local($line, %header, $s);
$s = $_[0];

# read headers
alarm(60);
($line = <$s>) =~ s/\r|\n//g;
if ($line !~ /^HTTP\/1\..\s+200\s+/) { &error("Download failed : $line"); }
while(<$s> =~ /^(\S+):\s+(.*)$/) { $header{lc($1)} = $2; }
alarm(0);

# read data
open(PFILE, "> $_[1]");
while(read($s, $buf, 1024) > 0) { print PFILE $buf; }
close(PFILE);
close($s);
}


# ftp_download(host, file, destfile)
# Download data from an FTP site to a local file
sub ftp_download
{
local($buf, @n);

$SIG{ALRM} = "download_timeout";
alarm(60);
if ($gconfig{'ftp_proxy'} =~ /^http:\/\/(\S+):(\d+)/ && !&no_proxy($_[0])) {
	# download through http-style proxy
	&open_socket($1, $2, "SOCK");
	print SOCK "GET ftp://$_[0]$_[1] HTTP/1.0\r\n";
	print SOCK "User-agent: Webmin\r\n";
	print SOCK "\r\n";
	&complete_http_download("SOCK", $_[2]);
	}
else {
	# connect to host and login
	&open_socket($_[0], 21, "SOCK");
	alarm(0);
	&ftp_command("", 2);
	&ftp_command("user anonymous", 3);
	&ftp_command("pass root\@".&get_system_hostname(), 2);

	# request the file
	&ftp_command("type i", 2);
	&ftp_command("pasv", 2) =~ /\(([0-9,]+)\)/;
	@n = split(/,/ , $1);
	&open_socket("$n[0].$n[1].$n[2].$n[3]", $n[4]*256 + $n[5], "CON");
	&ftp_command("retr $_[1]", 1);

	# transfer data
	open(PFILE, "> $_[2]");
	while(read(CON, $buf, 1024) > 0) { print PFILE $buf; }
	close(PFILE);
	close(CON);

	# finish off..
	&ftp_command("", 2);
	&ftp_command("quit", 2);
	close(SOCK);
	}
}

# no_proxy(host)
# Checks if some host is on the no proxy list
sub no_proxy
{
foreach $n (split(/\s+/, $gconfig{'noproxy'})) {
	if ($_[0] =~ /$n/) { return 1; }
	}
return 0;
}

# open_socket(host, port, handle)
sub open_socket
{
local($addr, $h); $h = $_[2];
socket($h, PF_INET, SOCK_STREAM, getprotobyname("tcp")) ||
	&error("Failed to create socket : $!");
($addr = inet_aton($_[0])) ||
	&error("Failed to lookup IP address for $_[0]");
connect($h, sockaddr_in($_[1], $addr)) ||
	&error("Failed to connect to $_[0]:$_[1] : $!");
select($h); $| =1; select(STDOUT);
}


# download_timeout()
# Called when a download times out
sub download_timeout
{
&error("Timeout downloading $in{url}");
}


# ftp_command(command, expected)
# Send an FTP command, and die if the reply is not what was expected
sub ftp_command
{
local($line, $code, $reply);
$what = $_[0] ne "" ? "<i>$_[0]</i>" : "initial connection";
if ($_[0] ne "") {
        print SOCK "$_[0]\r\n";
        }
alarm(60);
if (!($line = <SOCK>)) {
        &error("Failed to read reply to $what");
        }
$line =~ /^(...)(.)(.*)$/;
if (int($1/100) != $_[1]) {
        &error("$what failed : $3");
        }
$code = $1; $reply = $3;
if ($2 eq "-") {
        # Need to skip extra crap..
        while(1) {
                if (!($line = <SOCK>)) {
                        &error("Failed to read reply to $what");
                        }
                $line =~ /^(....)(.*)$/; $reply .= $2;
		if ($1 eq "$code ") { last; }
                }
        }
alarm(0);
return $reply;
}

# to_ipaddress(hostname)
# Converts a hostname to an a.b.c.d format IP address
sub to_ipaddress
{
local(@ip);
@ip = unpack("CCCC", gethostbyname($_[0]));
if (@ip) { return join("." , @ip); }
else { return undef; }
}

# icons_table(&links, &titles, &icons, [columns])
# Renders a 4-column table of icons
sub icons_table
{
local($i);
local $cols = $_[3] ? $_[3] : 4;
local $per = int(100.0 / $cols);
print "<CENTER><table width=620 cellpadding=0> <tr>\n";
for($i=0; $i<@{$_[0]}; $i++) {
	if ($i%$cols == 0) { print "<tr>\n"; }
	print "<td width=$per% align=center valign=top>\n";
	&generate_icon($_[2]->[$i], $_[1]->[$i], $_[0]->[$i]);
	print "</td>\n";
        if ($i%$cols == $cols-1) { print "</tr>\n"; }
        }
while($i++%$cols) { print "<td width=$per%></td>\n"; }
print "</table></CENTER><p>\n";
}

# replace_file_line(file, line, [newline]*)
# Replaces one line in some file with 0 or more new lines
sub replace_file_line
{
local(@lines);
open(FILE, $_[0]);
@lines = <FILE>;
close(FILE);
if (@_ > 2) { splice(@lines, $_[1], 1, @_[2..$#_]); }
else { splice(@lines, $_[1], 1); }
open(FILE, "> $_[0]");
print FILE @lines;
close(FILE);
}

# read_file_lines(file)
# Returns a reference to an array containing the lines from some file. This
# array can be modified, and will be written out when flush_file_lines()
# is called.
sub read_file_lines {
  if (!$file_cache{$_[0]}) {
    local(@lines, $_);

    open(READFILE, $_[0]);
      while(<READFILE>) { s/\r|\n//g; push(@lines, $_); }
    close(READFILE);

    $file_cache{$_[0]} = \@lines;
  }

  return $file_cache{$_[0]};
}

# flush_file_lines()
sub flush_file_lines {
  foreach $f (keys %file_cache) {
    open(FLUSHFILE, "> $f");
      foreach $line (@{$file_cache{$f}}) { print FLUSHFILE $line,"\n"; }
    close(FLUSHFILE);               
  }                               

  undef(%file_cache);
}                                       

# unix_user_input(fieldname, user)
# Returns HTML for an input to select a Unix user
sub unix_user_input
{
return "<input name=$_[0] size=8 value=\"$_[1]\"> ".
       &user_chooser_button($_[0], 0)."\n";
}

# unix_group_input(fieldname, user)
# Returns HTML for an input to select a Unix group
sub unix_group_input
{
return "<input name=$_[0] size=8 value=\"$_[1]\"> ".
       &group_chooser_button($_[0], 0)."\n";
}

# hlink(text, page, [module])
sub hlink
{
local $mod = $_[2] ? $_[2] : $module_name;
return "<a onClick='window.open(\"/help.cgi/$mod/$_[1]\", \"help\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300,resizable=yes\"); return false' href=\"/help.cgi/$mod/$_[1]\">$_[0]</a>";
}

# user_chooser_button(field, multiple, [form])
# Returns HTML for a javascript button for choosing a Unix user or users
sub user_chooser_button
{
  local $form = @_ > 2 ? $_[2] : 0;
  local $w = $_[1] ? 500 : 300;
  return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"/user_chooser.cgi?multi=$_[1]&user=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=$w,height=200\"); chooser.ifield = ifield;' value=\"...\">\n";
}

# group_chooser_button(field, multiple, [form])
# Returns HTML for a javascript button for choosing a Unix group or groups
sub group_chooser_button
{
local $form = @_ > 2 ? $_[2] : 0;
local $w = $_[1] ? 500 : 300;
return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"/group_chooser.cgi?multi=$_[1]&group=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=$w,height=200\"); chooser.ifield = ifield' value=\"...\">\n";
}

# foreign_check(module)
# Checks if some other module exists and is supported on this OS
sub foreign_check
{
local %minfo;
&read_file("../$_[0]/module.info", \%minfo) || return 0;
return &check_os_support(\%minfo);
}

# foreign_require(module, file)
# Brings in functions from another module
sub foreign_require
{
chdir("../$_[0]");
eval <<EOF;
package $_[0];
\$ENV{'FOREIGN_MODULE_NAME'} = '$_[0]';
do "./$_[1]";
EOF
if ($@) { &error($@); }
chdir("../$module_name");
return 1;
}

# foreign_call(module, function, [arg]*)
# Call a function in another module
sub foreign_call
{
chdir("../$_[0]");
local @args = @_[2 .. @_-1];
$main::foreign_args = \@args;
local @rv = eval <<EOF;
package $_[0];
&$_[1](\@{\$main::foreign_args});
EOF
if ($@) { &error($@); }
chdir("../$module_name");
return wantarray ? @rv : $rv[0];
}

# foreign_config(module)
# Get the configuration from another module
sub foreign_config
{
local %fconfig;
&read_file("$config_directory/$_[0]/config", \%fconfig);
return %fconfig;
}

# get_system_hostname()
# Returns the hostname of this system
sub get_system_hostname
{
if (!$get_system_hostname) {
	chop($get_system_hostname = `hostname 2>/dev/null`);
	if ($?) {
		use Sys::Hostname;
		$get_system_hostname = eval "hostname()";
		if ($@) { $get_system_hostname = "UNKNOWN"; }
		}
	}
return $get_system_hostname;
}

# get_webmin_version()
# Returns the version of Webmin currently being run
sub get_webmin_version {
  my ($version, $release) = (`/bin/rpm -q WebTool` =~ m/^WebTool\-(.*)\-(.*)$/);
  return "$version  r$release";
}


# get_module_acl([user], [module])
# Returns an array containing access control options for the given user
sub get_module_acl
{
local %rv;
local $u = $_[0] ? $_[0] : $ENV{'REMOTE_USER'};
local $m = $_[1] ? $_[1] : $module_name;

($mod_rwm) = @_; if ($mod_rwm) { $m = $mod_rwm; }

#&read_file("../$m/defaultacl", \%rv);

&read_file("/usr/webtool/$m/defaultacl", \%rv);
&read_file("$config_directory/$m/$u.acl", \%rv);

return %rv;
}

# save_module_acl(&acl, [user], [module])
# Updates the acl hash for some user and module (or the current one)
sub save_module_acl
{
local $u = $_[1] ? $_[1] : $ENV{'REMOTE_USER'};
local $m = $_[2] ? $_[2] : $module_name;
&write_file("$config_directory/$m/$u.acl", $_[0]);
}

# init_config()
# Sets the following variables
#  %config - Per-module configuration
#  %gconfig - Global configuration
#  $tb - Background for table headers
#  $cb - Background for table bodies
#  $scriptname - Base name of the current perl script
#  $module_name - The name of the current module
#  $module_config_directory - The config directory for this module
sub init_config {
  # Read the webmin global config file. This contains the OS type and version,
  # OS specific configuration and global options such as proxy servers
  &read_file("$config_directory/config", \%gconfig);

# Set PATH and LD_LIBRARY_PATH
$ENV{'PATH'} = $gconfig{'path'};
if ($gconfig{'ld_env'}) {
	$ENV{$gconfig{'ld_env'}} = $gconfig{'ld_path'};
	}

# Work out which module we are in, and read the per-module config file
if ($ENV{'FOREIGN_MODULE_NAME'}) {
	# In a foreign call - use the module name given
	$module_name = $ENV{'FOREIGN_MODULE_NAME'};
	}
elsif ($ENV{'SCRIPT_NAME'}) {
	if ($ENV{'SCRIPT_NAME'} =~ /^\/([^\/]+)\//) {
		# Get module name from CGI path
		$module_name = $1;
		}
	}
elsif ($0 =~ /([^\/]+)\/[^\/]+$/) {
	# Get module name from command line
	$module_name = $1;
	}
if ($module_name) {
	$module_config_directory = "$config_directory/$module_name";
	&read_file("$module_config_directory/config", \%config);
	%module_info = &get_module_info($module_name);
	}

# Set some useful variables
$tb = defined($gconfig{'cs_header'}) ? "bgcolor=#$gconfig{'cs_header'}"
				     : "bgcolor=#9999ff";
$cb = defined($gconfig{'cs_table'}) ? "bgcolor=#$gconfig{'cs_table'}"
				    : "bgcolor=#cccccc";
$0 =~ /([^\/]+)$/;
$scriptname = $1;

# Load language strings into %text
local $u = $ENV{'REMOTE_USER'};
$current_lang = $gconfig{"lang_$u"} ? $gconfig{"lang_$u"} :
		$gconfig{"lang"} ? $gconfig{"lang"} : $default_lang;
%text = &load_language();

# Check if the HTTP user can access this module
if ($module_name && !$main::no_acl_check && !$ENV{'FOREIGN_MODULE_NAME'}) {
	local(%acl, %minfo);
	&read_acl(\%acl, undef);
	%minfo = &get_module_info($module_name);
	$acl{$u,$module_name} || $acl{$u,'*'} ||
		&error("Access denied : User <i>$u</i> is not allowed to ".
		       "use the <i>$minfo{'desc'}</i> module");
	$main::no_acl_check++;
	}

# Check the Referer: header for nasty redirects
if ($0 && $0 !~ /index\.cgi$/ && $0 !~ /referer_save\.cgi$/ &&
    !$gconfig{'referer'} &&
    $ENV{'MINISERV_CONFIG'} &&
    $ENV{'HTTP_REFERER'} =~ /^http:\/\/([^\/]+)\// &&
    $1 ne $ENV{'HTTP_HOST'}) {
	# Looks like a link from elsewhere ..
	&header($text{'referer_title'}, "");
	print "<hr><center>\n";
	print "<form action=/referer_save.cgi>\n";
	&ReadParse();
	foreach $k (keys %in) {
		foreach $kk (split(/\0/, $in{$k})) {
			print "<input type=hidden name=$k value='$kk'>\n";
			}
		}
	print "<input type=hidden name=referer_original ",
	      "value='$ENV{'REQUEST_URI'}'>\n";

	$prot = lc($ENV{'HTTPS'}) eq 'on' ? "https" : "http";
	print "<p>",&text('referer_warn', "<tt>$ENV{'HTTP_REFERER'}</tt>",
	      "<tt>$prot://$ENV{'HTTP_HOST'}$ENV{'REQUEST_URI'}</tt>"),"<p>\n";
	print "<input type=submit value='$text{'referer_ok'}'><br>\n";
	print "<input type=checkbox name=referer_again value=1> ",
	      "$text{'referer_again'}<p>\n";
	print "</form></center><hr>\n";
	&footer("/", $text{'index'});
	exit;
	}

return 1;
}

$default_lang = "en";

# load_language([module])
# Returns a hashtable mapping text codes to strings in the appropriate language
sub load_language
{
local %text;
if ($module_name) {
	local $mod = $_[0] ? "../$_[0]" : ".";
	&read_file("../lang/$default_lang", \%text);
	&read_file("../lang/$current_lang", \%text);
	&read_file("$mod/lang/$default_lang", \%text);
	&read_file("$mod/lang/$current_lang", \%text);
	}
else {
	&read_file("lang/$default_lang", \%text);
	&read_file("lang/$current_lang", \%text);
	if ($_[0]) {
		&read_file("$_[0]/lang/$default_lang", \%text);
		&read_file("$_[0]/lang/$current_lang", \%text);
		}
	}
return %text;
}

# text(message, [substitute]+)
sub text
{
local $rv = $text{$_[0]};
local $i;
for($i=1; $i<@_; $i++) {
	$rv =~ s/\$$i/$_[$i]/g;
	}
return $rv;
}

# encode_base64(string)
# Encodes a string into base64 format
sub encode_base64
{
    local $res;
    pos($_[0]) = 0;                          # ensure start at the beginning
    while ($_[0] =~ /(.{1,45})/gs) {
        $res .= substr(pack('u', $1), 1)."\n";
        chop($res);
    }
    $res =~ tr|` -_|AA-Za-z0-9+/|;
    local $padding = (3 - length($_[0]) % 3) % 3;
    $res =~ s/.{$padding}$/'=' x $padding/e if ($padding);
    return $res;
}

# get_module_info(module, [noclone])
# Returns a hash containg a module name, desc and os_support
sub get_module_info
{
return () if ($_[0] =~ /^\./);
local (%rv, $clone);
if ($module_name && &read_file("../$_[0]/module.info", \%rv)) {
	$clone = -l "../$_[0]";
	}
elsif (&read_file("$_[0]/module.info", \%rv)) {
	$clone = -l $_[0];
	}
else { return (); }
$rv{"desc"} = $rv{"desc_$current_lang"} if ($rv{"desc_$current_lang"});
if ($clone && !$_[1]) {
	$rv{'clone'} = $rv{'desc'};
	&read_file("$config_directory/$_[0]/clone", \%rv);
	}
$rv{'dir'} = $_[0];
return %rv;
}

# list_languages()
# Returns an array of supported languages
sub list_languages
{
if (!@list_languages_cache) {
	local ($o, $_);
	open(LANG, "../lang_list.txt") || open(LANG, "lang_list.txt");
	while(<LANG>) {
		if (/^(\S+)\s+(.*)/) {
			local $l = { 'desc' => $2 };
			foreach $o (split(/,/, $1)) {
				if ($o =~ /^([^=]+)=(.*)$/) {
					$l->{$1} = $2;
					}
				}
			$l->{'index'} = scalar(@rv);
			push(@list_languages_cache, $l);
			}
		}
	close(LANG);
	}
return @list_languages_cache;
}

# read_env_file(file, &array)
sub read_env_file
{
open(FILE, $_[0]) || return 0;
while(<FILE>) {
	s/#.*$//g;
	if (/([A-z0-9_]+)\s*=\s*"(.*)"/ ||
	    /([A-z0-9_]+)\s*=\s*'(.*)'/ ||
	    /([A-z0-9_]+)\s*=\s*(\S+)/) {
		$_[1]->{$1} = $2;
		}
	}
close(FILE);
return 1;
}

# write_env_file(file, &array, export)
sub write_env_file
{
local $k;
local $exp = $_[2] ? "export " : "";
open(FILE, ">$_[0]");
foreach $k (keys %{$_[1]}) {
	local $v = $_[1]->{$k};
	if ($v =~ /^\S+$/) {
		print FILE "$exp$k=$v\n";
		}
	else {
		print FILE "$exp$k=\"$v\"\n";
		}
	}
close(FILE);
}

# lock_file(filename)
# Wait until a lock is obtained on the given file
sub lock_file
{
}

# unlock_file(filename)
# Release a lock grabbed by lock_file()
sub unlock_file
{
}

### ####################### ####################################################
### RWM DEFINED SUBROUTINES ####################################################
### ####################### ####################################################
sub ryanFooter {
  ($refPage)    =  @_;
  $gd_link      =  "http://www.guardiandigital.com/";
  $logout_link  =  "/session_login.cgi?logout=1";

  print <<FOOTER;
<TABLE WIDTH="100%" BORDER=0>
  <TR>
    <TD ALIGN="CENTER" VALIGN="CENTER">
      <IMG SRC="/images/foot.gif" USEMAP="#footer" BORDER="0">

      <MAP NAME="footer">
        <AREA HREF="$refPage" SHAPE="rect" COORDS="2,7,17,18">
      </MAP>

      <MAP NAME="header">
        <AREA HREF="$refPage" SHAPE="rect" COORDS="12,8,114,29">
        <AREA HREF="$hdr_url" SHAPE="rect" COORDS="114,8,314,29">
        <AREA HREF="/" SHAPE="rect" COORDS="314,8,420,29">
        <AREA HREF="$gd_link" TARGET="_gd" SHAPE="rect" COORDS="420,8,558,29">
        <AREA HREF="$logout_link" SHAPE="rect" COORDS="558,8,648,29">
      </MAP>

    </TD>
  </TR>
</TABLE>


</TD>
</TR>
</TABLE>

FOOTER

}


################################################################################
# Name:     lids_reload()
# Purpose:  Relod the configuration file.
################################################################################
sub lids_reload() {
  my $encpass  =  &get_lidspass();
  $out  =  `/usr/sbin/config_lids.pl --force 2>&1`;
  $out  = `/sbin/lidsadm -Q -- +RELOAD_CONF $encpass > /dev/null 2>&1`;
}


################################################################################
# Name:     lids_toggle()
# Purpose:  Turn LIDS on/off.
################################################################################
sub lids_toggle() {
  my ($onoff)  =  @_;
  my $encpass  =  &get_lidspass();

  if ($onoff eq "on") {
    $out = `/sbin/lidsadm_gd -Q -- +LIDS_GLOBAL $encpass > /dev/null 2>&1`;
  }
  elsif ($onoff eq "off") {
    $out = `/sbin/lidsadm_gd -Q -- -LIDS_GLOBAL $encpass > /dev/null 2>&1`;
  }
}


################################################################################
# Name:     get_lidspass()
# Purpose:  Retrieve the password for use by reload() or toggle().
################################################################################
sub get_lidspass() {
  use Crypt::RIPEMD160;

  ### Get the password and hash it.
  open FILE, "/etc/webtool/SystemManager/config";
    while (<FILE>) { if ($_ =~ m/^pass\=(.*)$/) { $lids_pass = $1; } }
  close FILE;

  ### Okay, now we have our password.  Next we must disable LIDS.
  my $hash        =  new Crypt::RIPEMD160;
  $hash->add($lids_pass);
  my $digest      =  $hash->digest();;
  my $encpass     =  unpack "H*", $digest;

  return $encpass;
}


###
### Example Usage of clean_env() and restore_env()
###
###    ### Clean and save the environment.
###    my %ENV_RESTORE;
###    %ENV_RESTORE  =  &clean_env();
###
###      ### Restart the daemon.
###      $out1 = `/etc/init.d/httpd restart 2>&1`;
###
###    ### Restore the environment.
###    &restore_env(%ENV_RESTORE);
###

################################################################################
# Name:     clean_env()
# Purpose:  Clear out all environmental varibles and store copy in hash passed.
################################################################################
sub clean_env()
{
  my (%ENV_RESTORE);

  foreach my $k (keys %ENV) { $ENV_RESTORE{$k} = $ENV{$k}; delete($ENV{$k}); }

  return %ENV_RESTORE;
}

################################################################################
# Name:     restore_env()
# Purpose:  Restore the environemnt which was cleared with clean_env from
#           hash passed.
################################################################################
sub restore_env()
{
  my (%ENV_RESTORE)  =  @_;

  foreach my $k (keys %ENV_RESTORE) { $ENV{$k} = $ENV_RESTORE{$k}; }
}



### unlock_all_files()
### Unlocks all files locked by this program
sub unlock_all_files {
  foreach $f (keys %main::locked_file_list) { &unlock_file($f); }
}



1;  # return true?

