#!/usr/bin/perl

  ### ########################################################
  ### Include the standard library and define local variables.
  ### ########################################################
  require 'sshd-lib.pl';
  require '../../Security/Security.pl';
  my ($username, $keysize, $comment, $passphrase1, $passphrase2);
  my ($sshkeygen, $output);


  ### #####################################
  ### Let's give our variables some values.
  ### #####################################
  &ReadParse();					### Process CGI.
  $sshkeygen = $config{'ssh-keygen'};		### Location of ssh-keygen

  $username    = $in{'Username'};		### Username.
  $ip_address  = $in{'IPAddress'};		### Address for hosts.allow.
  $keysize     = $in{'KeySize'};		### Size of key.
  $comment     = $in{'Comment'};		### Key comment.
  $passphrase1 = $in{'Passphrase1'};		### Passphrase.
  $passphrase2 = $in{'Passphrase2'};		### Passphrase verification.



  ### ######################################################################
  ### Let's make sure all our variables are there and that $username exists.
  ### ######################################################################
  if (!$username) {
    &PrintError("Please enter an username.");
    exit;
  }
  elsif (!&ValidUser($username)) {
    &PrintError("$username does not exist.  Please enter an existing user.");
    exit;
  }
  elsif (!$comment) {
    &PrintError("Please enter a comment to identify this key pair.");
    exit;
  }
  elsif (!$passphrase1 || !passphrase2) {
    &PrintError("Please enter a passphrase.");
    exit;
  }
  elsif ($passphrase1 ne $passphrase2) {
    &PrintError("The two passphrases do not match.  Please re-enter.");
    exit;
  }



  ### If we have an address, and it is valid, put it in /etc/hosts.allow.
  if ($ip_address && !&check_ipaddress($ip_address)) {
    &PrintError("'$ipaddress' is not a valid IP address.");
    exit;
  }
  elsif (ip_address) { &AddACL($ip_address, "Allow", "sshd"); }



  ### #######################################################################
  ### We have everything we need, so let's go ahead and generate the keypair.
  ### #######################################################################
  if ($username eq "root") {
    ### SANITY CHECK THE INPUT STRINGS for "'s
    $comment     =~ s/\"/\\\"/g;
    $passphrase1 =~ s/\"/\\\"/g;

    ### If root's .ssh directory does not exist, create it.
    system "/bin/mkdir -m 700 -p /root/.ssh";

    ### Remove the old keys if they exist.    
    unlink "/bin/rm /root/.ssh/identity_webtool";
    unlink "/bin/rm /root/.ssh/identity_webtool.pub";

    ### Execute ssh-keygen.
    $output = `$sshkeygen -b $keysize -f /root/.ssh/identity_webtool -N \"$passphrase1\" -C \"$comment\"`;

    ### Put the key in /root/.ssh/authorized_keys.
    system "cat /root/.ssh/identity_webtool.pub >> /root/.ssh/authorized_keys";

    ### Make the key files readable only by the owner.
    system "/bin/chmod 400 /root/.ssh/identity_webtool";
    system "/bin/chmod 400 /root/.ssh/identity_webtool.pub";
    system "/bin/chmod 600 /root/.ssh/authorized_keys";

    $KEYFILE = "/root/.ssh/identity_webtool";
  }

  ### We have a normal user.
  else {
    ### Create the directory if it does not exist.
    system "/bin/mkdir -m 700 -p /home/$username/.ssh";

    ### SANITY CHECK THE INPUT STRINGS for "'s
    $comment     =~ s/\"/\\\"/g;
    $passphrase1 =~ s/\"/\\\"/g;
  
    ### Remove the old keys if they exist.
    unlink "/bin/rm /home/$username/.ssh/identity_webtool";
    unlink "/bin/rm /home/$username/.ssh/identity_webtool.pub";

    ### Execute ssh-keygen.
    $output = `$sshkeygen -b $keysize -f /home/$username/.ssh/identity_webtool -N \"$passphrase1\" -C \"$comment\"`;

    ### Put the key in ~/.ssh/authorized_keys.
    system "cat /home/$username/.ssh/identity_webtool.pub >> /home/$username/.ssh/authorized_keys";

    ### Make the key files readable only by the owner.
    system "/bin/chmod 400 /home/$username/.ssh/identity_webtool";
    system "/bin/chmod 400 /home/$username/.ssh/identity_webtool.pub";
    system "/bin/chmod 600 /home/$username/.ssh/authorized_keys";

      ### Find the user's GID.
      setpwent();
        while(@tmp = getpwent()) { if ($tmp[0] eq $user) { $gid = $tmp[3]; } }
      endpwent();

      ### Find the user's group.
      setgrent();
        while(@tmp = getgrent()) { if ($tmp[2] eq $gid) { $group = $tmp[0]; } }
      endgrent();

    ### Make sure the .ssh directory and it's contents are owned by $username.
    system "/bin/chown -R $username.$group /home/$username/.ssh/";

    $KEYFILE = "/home/$username/.ssh/identity_webtool";
  }

  ### Print the header and a horizonal rule.
  &header("Key Generation Complete", "");

  print <<EOF01;
<FORM ACTION="download_key.cgi" METHOD="POST">
<CENTER><TABLE WIDTH="620" CELLPADDING=0 CELLSPACING=0 BORDER=0>
  <TR $tb> <TD COLSPAN=2><DIV CLASS=ttitle><B>SSH Key Generation Complete</TD></TR>

  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>

  <TR $cb>
    <TD ALIGN="CENTER"><DIV CLASS=nbody>
    <INPUT TYPE="HIDDEN" NAME="keyfile" VALUE="$KEYFILE">
    <INPUT TYPE="HIDDEN" NAME="username" VALUE="$username">
    <INPUT TYPE="SUBMIT" VALUE="Click Here to Download Private Key">
      <BR><B>(Save this to your computer and name it something you will 
             remember.)
    </TD>
  </TR>

  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
</TABLE></CENTER></FORM><P>
EOF01


  ### Close up and print the footer.
  &ryanFooter("");
